@@ -13,6 +13,7 @@
 (in-package "SB-IMPL")
 
 ;;; HASH-TABLE is implemented as a STRUCTURE-OBJECT.
+(sb-xc:deftype hash-table-index () '(unsigned-byte 32))
 (sb-xc:defstruct (hash-table (:copier nil)
                              (:constructor %make-hash-table
                                (test
@@ -39,10 +40,10 @@
   ;; How much to grow the hash table by when it fills up. If an index,
   ;; then add that amount. If a floating point number, then multiply
   ;; it by that.
-  (rehash-size nil :type (or index (single-float (1.0)))
+  (rehash-size nil :type (or index (single-float ($1.0)))
                :read-only t)
   ;; How full the hash table has to get before we rehash
-  (rehash-threshold nil :type (single-float (0.0) 1.0) :read-only t)
+  (rehash-threshold nil :type (single-float ($0.0) $1.0) :read-only t)
   ;; The number of entries before a rehash, just one less than the
   ;; size of the next-vector, hash-vector, and half the size of the
   ;; kv-vector.
@@ -65,18 +66,18 @@
   (cache nil :type (or null index))
   ;; The index vector. This may be larger than the hash size to help
   ;; reduce collisions.
-  (index-vector nil :type (simple-array sb-vm:word (*)))
+  (index-vector nil :type (simple-array hash-table-index (*)))
   ;; This table parallels the KV vector, and is used to chain together
   ;; the hash buckets and the free list. A slot will only ever be in
   ;; one of these lists.
-  (next-vector nil :type (simple-array sb-vm:word (*)))
+  (next-vector nil :type (simple-array hash-table-index (*)))
   ;; This table parallels the KV table, and can be used to store the
   ;; hash associated with the key, saving recalculation. Could be
   ;; useful for EQL, and EQUAL hash tables. This table is not needed
   ;; for EQ hash tables, and when present the value of
   ;; +MAGIC-HASH-VECTOR-VALUE+ represents EQ-based hashing on the
   ;; respective key.
-  (hash-vector nil :type (or null (simple-array sb-vm:word (*))))
+  (hash-vector nil :type (or null (simple-array hash-table-index (*))))
   ;; Used for locking GETHASH/(SETF GETHASH)/REMHASH
   (lock (sb-thread:make-mutex :name "hash-table lock")
         #-c-headers-only :type #-c-headers-only sb-thread:mutex
@@ -91,13 +92,14 @@
   #+sb-hash-table-debug
   (writing-thread nil))
 
-;; as explained by pmai on openprojects #lisp IRC 2002-07-30: #x80000000
-;; is bigger than any possible nonEQ hash value, and thus indicates an
-;; empty slot; and EQ hash tables don't use HASH-TABLE-HASH-VECTOR.
-;; The previous sentence was written when SBCL was 32-bit only. The value
-;; now depends on the word size. It is propagated to C in genesis because
-;; the generational garbage collector needs to know it.
-(defconstant +magic-hash-vector-value+ (ash 1 (1- sb-vm:n-word-bits)))
+;; Our hash-tables store precomputed hashes to speed rehash and to guard
+;; the call of the general comparator.
+;; i.e. we take the value from mumble-hash {SXHASH, EQ-HASH, etc}
+;; and fuzz the bits some more, then clip to 31 bits and store that.
+;; (As a practical matter, this limits tables to 2^31 bins.)
+;; Address-sensitive keys can't store a precomputed hash. They instead
+;; store this value that indicates address-sensitivity.
+(defconstant +magic-hash-vector-value+ #xFFFFFFFF)
 
 ;;; Return an association list representing the same data as HASH-TABLE.
 (defun %hash-table-alist (hash-table)
