@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB-IMPL") ;(SB-IMPL, not SB!IMPL, since we're built in warm load.)
+(in-package "SB-IMPL")
 
 ;;;; hacking the Unix environment
 ;;;;
@@ -103,8 +103,6 @@
        (concatenate 'simple-string (symbol-name key) "=" val)))
    cmucl))
 
-;;;; Import wait3(2) from Unix.
-
 #-win32
 (define-alien-routine ("waitpid" c-waitpid) int
   (pid int)
@@ -112,32 +110,29 @@
   (options int))
 
 #-win32
-(defun waitpid (pid &optional do-not-hang check-for-stopped)
+(defun waitpid (pid)
   "Return any available status information on child process with PID."
   (multiple-value-bind (pid status)
       (c-waitpid pid
-                 (logior (if do-not-hang
-                             sb-unix:wnohang
-                             0)
-                         (if check-for-stopped
-                             sb-unix:wuntraced
-                             0)))
+                 (logior sb-unix:wnohang sb-unix:wuntraced sb-unix:wcontinued))
     (cond ((or (minusp pid)
                (zerop pid))
-           nil)
-          ((eql (ldb (byte 8 0) status)
-                sb-unix:wstopped)
-           (values pid
-                   :stopped
-                   (ldb (byte 8 8) status)))
+           (values nil nil nil))
+          ((wifcontinued status)
+           (values :running
+                   nil
+                   nil))
+          ((wifstopped status)
+           (values :stopped
+                   (ldb (byte 8 8) status)
+                   nil))
           ((zerop (ldb (byte 7 0) status))
-           (values pid
-                   :exited
-                   (ldb (byte 8 8) status)))
+           (values :exited
+                   (ldb (byte 8 8) status)
+                   nil))
           (t
            (let ((signal (ldb (byte 7 0) status)))
-             (values pid
-                     (if (position signal
+             (values (if (position signal
                                    #.(vector
                                       sb-unix:sigstop
                                       sb-unix:sigtstp
@@ -147,14 +142,25 @@
                          :signaled)
                      signal
                      (not (zerop (ldb (byte 1 7) status)))))))))
+
+#-win32
+(define-alien-routine wifcontinued boolean
+  (status int))
+
+#-win32
+(define-alien-routine wifstopped boolean
+  (status int))
 
 ;;;; process control stuff
-(defvar *active-processes* nil
+(define-load-time-global *active-processes* nil
   "List of process structures for all active processes.")
 
-(defvar *active-processes-lock*
+(define-load-time-global *active-processes-lock*
   (sb-thread:make-mutex :name "Lock for active processes."))
 
+(define-load-time-global *spawn-lock*
+  (sb-thread:make-mutex :name "Around spawn()."))
+
 ;;; *ACTIVE-PROCESSES* can be accessed from multiple threads so a
 ;;; mutex is needed. More importantly the sigchld signal handler also
 ;;; accesses it, that's why we need without-interrupts.
@@ -162,27 +168,32 @@
   `(sb-thread::with-system-mutex (*active-processes-lock*)
      ,@body))
 
+(deftype process-status ()
+  '(member :running :stopped :exited :signaled))
+
 (defstruct (process (:copier nil))
-  pid                 ; PID of child process
-  %status             ; either :RUNNING, :STOPPED, :EXITED, or :SIGNALED
-  %exit-code          ; either exit code or signal
-  core-dumped         ; T if a core image was dumped
-  #-win32 pty         ; stream to child's pty, or NIL
-  input               ; stream to child's input, or NIL
-  output              ; stream from child's output, or NIL
-  error               ; stream from child's error output, or NIL
-  status-hook         ; closure to call when PROC changes status
-  plist               ; a place for clients to stash things
-  cookie              ; list of the number of pipes from the subproc
-  #+win32
-  copiers)            ; list of sb-win32::io-copier
+  (pid     nil :type word :read-only t) ; PID of child process
+  (%status nil :type process-status)
+  %exit-code                            ; either exit code or signal
+  core-dumped                           ; T if a core image was dumped
+  #-win32 pty                           ; stream to child's pty, or NIL
+  input                                 ; stream to child's input, or NIL
+  output                                ; stream from child's output, or NIL
+  error                                 ; stream from child's error output, or NIL
+  status-hook                           ; closure to call when PROC changes status
+  plist                                 ; a place for clients to stash things
+  (cookie nil :type cons :read-only t)  ; list of the number of pipes from the subproc
+  #+win32 copiers ; list of sb-win32::io-copier
+  #+win32 (handle nil :type (or null (signed-byte 32)))
+  #-win32
+  serve-event-pipe)
 
 (defmethod print-object ((process process) stream)
   (print-unreadable-object (process stream :type t)
     (let ((status (process-status process)))
-     (if (eq :exited status)
-         (format stream "~S ~S" status (process-%exit-code process))
-         (format stream "~S ~S" (process-pid process) status)))
+      (if (eq :exited status)
+          (format stream "~S ~S" status (process-%exit-code process))
+          (format stream "~S ~S" (process-pid process) status)))
     process))
 
 #+win32
@@ -222,6 +233,37 @@ The function is called with PROCESS as its only argument."
       "T if OBJECT is a PROCESS, NIL otherwise."
       (documentation 'process-pid 'function) "The pid of the child process.")
 
+#-win32
+(defun setup-serve-event-pipe (process)
+  (unless (process-serve-event-pipe process)
+    (multiple-value-bind (read-fd write-fd) (sb-unix:unix-pipe)
+      (let (handler)
+        (setf handler
+              (add-fd-handler read-fd :input
+                              (lambda (fd)
+                                (setf (process-serve-event-pipe process) nil)
+                                (remove-fd-handler handler)
+                                (sb-unix:unix-close fd))))
+        (setf (process-serve-event-pipe process) (list* read-fd write-fd handler))))))
+
+#-win32
+(defun close-serve-event-pipe (process)
+  (let ((pipe (process-serve-event-pipe process)))
+    (when pipe
+      (setf (process-serve-event-pipe process) nil)
+      (destructuring-bind (read write . handler) pipe
+        (remove-fd-handler handler)
+        (sb-unix:unix-close read)
+        (when write
+          (sb-unix:unix-close write))))))
+
+(defun wake-serve-event (process)
+  #+win32 (declare (ignorable process))
+  #-win32
+  (let ((pipe (process-serve-event-pipe process)))
+    (when pipe
+      (sb-unix:unix-close (shiftf (cadr pipe) nil)))))
+
 (defun process-wait (process &optional check-for-stopped)
   "Wait for PROCESS to quit running for some reason. When
 CHECK-FOR-STOPPED is T, also returns when PROCESS is stopped. Returns
@@ -230,16 +272,19 @@ PROCESS."
   #+win32
   (sb-win32::win32-process-wait process)
   #-win32
-  (loop
-      (case (process-status process)
-        (:running)
-        (:stopped
-         (when check-for-stopped
-           (return)))
-        (t
-         (when (zerop (car (process-cookie process)))
-           (return))))
-      (serve-all-events 1))
+  (progn
+    (setup-serve-event-pipe process)
+    (loop
+     (case (process-status process)
+       (:running)
+       (:stopped
+        (when check-for-stopped
+          (return)))
+       (t
+        (when (zerop (car (process-cookie process)))
+          (return))))
+     (serve-all-events 10))
+    (close-serve-event-pipe process))
   process)
 
 #-win32
@@ -270,17 +315,17 @@ PROCESS."
     (let ((result (if (eq whom :process-group)
                       (sb-unix:unix-killpg pid signal)
                       (sb-unix:unix-kill pid signal))))
-      (cond ((not (eql result 0))
-             (values nil (sb-unix::get-errno)))
-            ((and (eql pid (process-pid process))
-                  (= signal sb-unix:sigcont))
-             (setf (process-%status process) :running)
-             (setf (process-%exit-code process) nil)
-             (when (process-status-hook process)
-               (funcall (process-status-hook process) process))
-             t)
-            (t
-             t)))))
+      (or (zerop result)
+          (values nil (sb-unix::get-errno))))))
+
+#+win32
+(defun process-kill (process signal &optional (whom :pid))
+  (declare (ignore signal whom))
+  (get-processes-status-changes)
+  (let ((handle (process-handle process)))
+    (when handle
+      (prog1 (sb-win32::terminate-process handle 1)
+        (get-processes-status-changes)))))
 
 (defun process-alive-p (process)
   "Return T if PROCESS is still alive, NIL otherwise."
@@ -305,14 +350,14 @@ status slot."
   (with-active-processes-lock ()
    (setf *active-processes* (delete process *active-processes*)))
   #+win32
-  (let ((handle (shiftf (process-pid process) nil)))
-    (when (and handle (plusp handle))
-      (or (sb-win32:close-handle handle)
+  (let ((handle (shiftf (process-handle process) nil)))
+    (when handle
+      (or (plusp (sb-win32:close-handle handle))
           (sb-win32::win32-error 'process-close))))
   process)
 
 (defun get-processes-status-changes ()
-  (let (exited)
+  (let (changed)
     (with-active-processes-lock ()
       (setf *active-processes*
             (delete-if #-win32
@@ -322,37 +367,40 @@ status slot."
                          ;; WAIT3 call here, but that makes direct
                          ;; WAIT, WAITPID usage impossible due to the
                          ;; race with the SIGCHLD signal handler.
-                         (multiple-value-bind (pid what code core)
-                             (waitpid (process-pid proc) t t)
-                           (when pid
-                             (setf (process-%status proc) what)
+                         (multiple-value-bind (status code core)
+                             (waitpid (process-pid proc))
+                           (when status
+                             (wake-serve-event proc)
+                             (setf (process-%status proc) status)
                              (setf (process-%exit-code proc) code)
-                             (setf (process-core-dumped proc) core)
                              (when (process-status-hook proc)
-                               (push proc exited))
-                             t)))
+                               (push proc changed))
+                             (when (member status '(:exited :signaled))
+                               (setf (process-core-dumped proc) core)
+                               t))))
                        #+win32
                        (lambda (proc)
-                         (let ((pid (process-pid proc)))
-                           (when pid
+                         (let ((handle (process-handle proc)))
+                           (when handle
                              (multiple-value-bind (ok code)
-                                 (sb-win32::get-exit-code-process pid)
+                                 (sb-win32::get-exit-code-process handle)
                                (when (and (plusp ok)
                                           (/= code sb-win32::still-active))
                                  (setf (process-%status proc) :exited
                                        (process-%exit-code proc) code)
+                                 (sb-win32::close-handle handle)
+                                 (setf (process-handle proc) nil)
                                  (when (process-status-hook proc)
-                                   (push proc exited))
+                                   (push proc changed))
                                  t)))))
                        *active-processes*)))
     ;; Can't call the hooks before all the processes have been deal
     ;; with, as calling a hook may cause re-entry to
-    ;; GET-PROCESS-STATUS-CHANGES. That may be OK when using waitpid,
+    ;; GET-PROCESSES-STATUS-CHANGES. That may be OK when using waitpid,
     ;; but in the Windows implementation it would be deeply bad.
-    (dolist (proc exited)
+    (dolist (proc changed)
       (let ((hook (process-status-hook proc)))
-        (when hook
-          (funcall hook proc))))))
+        (funcall hook proc)))))
 
 ;;;; RUN-PROGRAM and close friends
 
@@ -376,11 +424,10 @@ status slot."
 
   (defun find-a-pty ()
     ;; First try to use the Unix98 pty api.
-    (let* ((master-name (coerce (format nil "/dev/ptmx") 'base-string))
-           (master-fd (sb-unix:unix-open master-name
-                                         (logior sb-unix:o_rdwr
-                                                 sb-unix:o_noctty)
-                                         #o666)))
+    (let ((master-fd (sb-unix:unix-open "/dev/ptmx"
+                                        (logior sb-unix:o_rdwr
+                                                sb-unix:o_noctty)
+                                        #o666)))
       (when master-fd
         (grantpt master-fd)
         (unlockpt master-fd)
@@ -399,15 +446,15 @@ status slot."
     ;; No dice, try using the old-school method.
     (dolist (char '(#\p #\q))
       (dotimes (digit 16)
-        (let* ((master-name (coerce (format nil "/dev/pty~C~X" char digit)
-                                    'base-string))
+        (let* ((master-name (with-simple-output-to-string (str nil base-char)
+                              (format str "/dev/pty~C~X" char digit)))
                (master-fd (sb-unix:unix-open master-name
                                              (logior sb-unix:o_rdwr
                                                      sb-unix:o_noctty)
                                              #o666)))
           (when master-fd
-            (let* ((slave-name (coerce (format nil "/dev/tty~C~X" char digit)
-                                       'base-string))
+            (let* ((slave-name (with-simple-output-to-string (str nil base-char)
+                                 (format str "/dev/tty~C~X" char digit)))
                    (slave-fd (sb-unix:unix-open slave-name
                                                 (logior sb-unix:o_rdwr
                                                         sb-unix:o_noctty)
@@ -439,22 +486,24 @@ status slot."
 #-win32
 (defun open-pty (pty cookie &key (external-format :default))
   (when pty
-    (multiple-value-bind
-          (master slave name)
-        (find-a-pty)
-      (push master *close-on-error*)
-      (push slave *close-in-parent*)
-      (when (streamp pty)
-        (multiple-value-bind (new-fd errno) (sb-unix:unix-dup master)
-          (unless new-fd
-            (error "couldn't SB-UNIX:UNIX-DUP ~W: ~A" master (strerror errno)))
-          (push new-fd *close-on-error*)
-          (copy-descriptor-to-stream new-fd pty cookie external-format)))
-      (values name
-              (make-fd-stream master :input t :output t
-                                     :external-format external-format
-                                     :element-type :default
-                                     :dual-channel-p t)))))
+    ;; ptsname is not thread-safe, ptsname_r is.
+    (with-active-processes-lock ()
+      (multiple-value-bind
+            (master slave name)
+          (find-a-pty)
+        (push master *close-on-error*)
+        (push slave *close-in-parent*)
+        (when (streamp pty)
+          (multiple-value-bind (new-fd errno) (sb-unix:unix-dup master)
+            (unless new-fd
+              (error "couldn't SB-UNIX:UNIX-DUP ~W: ~A" master (strerror errno)))
+            (push new-fd *close-on-error*)
+            (copy-descriptor-to-stream new-fd pty cookie external-format)))
+        (values name
+                (make-fd-stream master :input t :output t
+                                       :external-format external-format
+                                       :element-type :default
+                                       :dual-channel-p t))))))
 
 ;; Null terminate strings only C-side: otherwise we can run into
 ;; A-T-S-L even for simple encodings like ASCII.  Multibyte encodings
@@ -538,9 +587,15 @@ status slot."
   (search int)
   (envp (* c-string))
   (pty-name c-string)
-  (wait int)
+  (channel (array int 2))
   (dir c-string))
 
+#-win32
+(define-alien-routine wait-for-exec
+  int
+  (pid int)
+  (channel (array int 2)))
+
 #+win32
 (defun escape-arg (arg stream)
   ;; Normally, #\\ doesn't have to be escaped
@@ -656,14 +711,8 @@ status slot."
                     (external-format :default)
                     directory
                     #+win32 (escape-arguments t))
-  #.(concatenate
-     'base-string
-     ;; The Texinfoizer is sensitive to whitespace, so mind the
-     ;; placement of the #-win32 pseudosplicings.
-     "RUN-PROGRAM creates a new process specified by the PROGRAM
-argument. ARGS are the standard arguments that can be passed to a
-program. For no arguments, use NIL (which means that just the
-name of the program is passed as arg 0).
+  "RUN-PROGRAM creates a new process specified by PROGRAM.
+ARGS are passed as the arguments to the program.
 
 The program arguments and the environment are encoded using the
 default external format for streams.
@@ -675,13 +724,13 @@ Users Manual for details about the PROCESS structure.
 
    - The SBCL implementation of RUN-PROGRAM, like Perl and many other
      programs, but unlike the original CMU CL implementation, copies
-     the Unix environment by default."#-win32"
+     the Unix environment by default.
    - Running Unix programs from a setuid process, or in any other
      situation where the Unix environment is under the control of someone
      else, is a mother lode of security problems. If you are contemplating
      doing this, read about it first. (The Perl community has a lot of good
      documentation about this and other security issues in script-like
-     programs.)""
+     programs.)
 
    The &KEY arguments have the following meanings:
    :ENVIRONMENT
@@ -696,43 +745,50 @@ Users Manual for details about the PROCESS structure.
       environment variable.  Otherwise an absolute pathname is required.
    :WAIT
       If non-NIL (default), wait until the created process finishes.  If
-      NIL, continue running Lisp until the program finishes."#-win32"
-   :PTY
+      NIL, continue running Lisp until the program finishes.
+   :PTY (not supported on win32)
       Either T, NIL, or a stream.  Unless NIL, the subprocess is established
       under a PTY.  If :pty is a stream, all output to this pty is sent to
       this stream, otherwise the PROCESS-PTY slot is filled in with a stream
-      connected to pty that can read output and write input.""
+      connected to pty that can read output and write input.
    :INPUT
-      Either T, NIL, a pathname, a stream, or :STREAM.  If T, the standard
-      input for the current process is inherited.  If NIL, "
-      #-win32"/dev/null"#+win32"nul""
-      is used.  If a pathname, the file so specified is used.  If a stream,
-      all the input is read from that stream and sent to the subprocess.  If
-      :STREAM, the PROCESS-INPUT slot is filled in with a stream that sends
-      its output to the process. Defaults to NIL.
+      Either T, NIL, a pathname, a stream, or :STREAM.
+      T: the standard input for the current process is inherited.
+      NIL: /dev/null (nul on win32) is used.
+      pathname: the specified file is used.
+      stream: all the input is read from that stream and sent to the
+      subprocess.
+      :STREAM: the PROCESS-INPUT slot is filled in with a stream that sends
+      its output to the process.
+      Defaults to NIL.
    :IF-INPUT-DOES-NOT-EXIST (when :INPUT is the name of a file)
       can be one of:
          :ERROR to generate an error
          :CREATE to create an empty file
          NIL (the default) to return NIL from RUN-PROGRAM
    :OUTPUT
-      Either T, NIL, a pathname, a stream, or :STREAM.  If T, the standard
-      output for the current process is inherited.  If NIL, "
-      #-win32"/dev/null"#+win32"nul""
-      is used.  If a pathname, the file so specified is used.  If a stream,
-      all the output from the process is written to this stream. If
-      :STREAM, the PROCESS-OUTPUT slot is filled in with a stream that can
-      be read to get the output. Defaults to NIL.
+      Either T, NIL, a pathname, a stream, or :STREAM.
+      T: the standard output for the current process is inherited.
+      NIL: /dev/null (nul on win32) is used.
+      pathname: the specified file is used.
+      stream: all the output from the process is written to this stream.
+      :STREAM: the PROCESS-OUTPUT slot is filled in with a stream that can be
+      read to get the output.
+      Defaults to NIL.
+   :ERROR
+      Same as :OUTPUT, additionally accepts :OUTPUT, making all error
+      output routed to the same place as normal output.
+      Defaults to :OUTPUT.
    :IF-OUTPUT-EXISTS (when :OUTPUT is the name of a file)
       can be one of:
          :ERROR (the default) to generate an error
          :SUPERSEDE to supersede the file with output from the program
          :APPEND to append output from the program to the file
          NIL to return NIL from RUN-PROGRAM, without doing anything
-   :ERROR and :IF-ERROR-EXISTS
-      Same as :OUTPUT and :IF-OUTPUT-EXISTS, except that :ERROR can also be
-      specified as :OUTPUT in which case all error output is routed to the
-      same place as normal output.
+   :IF-ERROR-EXISTS
+      Same as :IF-OUTPUT-EXISTS, controlling :ERROR output to files.
+      Ignored when :ERROR :OUTPUT.
+      Defaults to :ERROR.
    :STATUS-HOOK
       This is a function the system calls whenever the status of the
       process changes.  The function takes the process as an argument.
@@ -744,7 +800,7 @@ Users Manual for details about the PROCESS structure.
 
    Windows specific options:
    :ESCAPE-ARGUMENTS (default T)
-      Controls escaping of the arguments passed to CreateProcess.")
+      Controls escaping of the arguments passed to CreateProcess."
   (when (and env-p environment-p)
     (error "can't specify :ENV and :ENVIRONMENT simultaneously"))
   (let* (;; Clear various specials used by GET-DESCRIPTOR-FOR to
@@ -804,53 +860,65 @@ Users Manual for details about the PROCESS structure.
                                         :direction :output
                                         :if-exists if-error-exists
                                         :external-format external-format)
-                 (with-open-pty ((pty-name pty-stream) (pty cookie))
-                   ;; Make sure we are not notified about the child
-                   ;; death before we have installed the PROCESS
-                   ;; structure in *ACTIVE-PROCESSES*.
-                   (let (child)
-                     (with-active-processes-lock ()
-                       (with-environment (environment-vec environment
-                                          :null (not (or environment environment-p)))
-                         (setq child
+                 ;; Make sure we are not notified about the child
+                 ;; death before we have installed the PROCESS
+                 ;; structure in *ACTIVE-PROCESSES*.
+                 (let (child #+win32 handle)
+                   (with-environment (environment-vec environment
+                                      :null (not (or environment environment-p)))
+                     (with-alien (#-win32 (channel (array int 2)))
+                       (with-open-pty ((pty-name pty-stream) (pty cookie))
+                         (setf (values child #+win32 handle)
                                #+win32
-                               (sb-win32::mswin-spawn
-                                progname
-                                args
-                                stdin stdout stderr
-                                search environment-vec wait directory)
+                               (sb-thread::with-system-mutex (*spawn-lock*)
+                                 (sb-win32::mswin-spawn
+                                  progname
+                                  args
+                                  stdin stdout stderr
+                                  search environment-vec directory))
                                #-win32
                                (with-args (args-vec args)
-                                 (without-gcing
+                                 (sb-thread::with-system-mutex (*spawn-lock*)
                                    (spawn progname args-vec
                                           stdin stdout stderr
                                           (if search 1 0)
                                           environment-vec pty-name
-                                          (if wait 1 0) directory))))
+                                          channel
+                                          directory))))
                          (unless (minusp child)
-                           (setf proc
-                                 (make-process
-                                  :input input-stream
-                                  :output output-stream
-                                  :error error-stream
-                                  :status-hook status-hook
-                                  :cookie cookie
-                                  #-win32 :pty #-win32 pty-stream
-                                  :%status :running
-                                  :pid child
-                                  #+win32 :copiers #+win32 *handlers-installed*))
-                           (push proc *active-processes*))))
-                     ;; Report the error outside the lock.
-                     (case child
-                       (-1
-                        (error "Couldn't fork child process: ~A"
-                               (strerror)))
-                       (-2
-                        (error "Couldn't execute ~S: ~A"
-                               progname (strerror)))
-                       (-3
-                        (error "Couldn't change directory to ~S: ~A"
-                               directory (strerror))))))))))
+                           #-win32
+                           (setf child (wait-for-exec child channel))
+                           (unless (minusp child)
+                             (setf proc
+                                   (make-process
+                                    :input input-stream
+                                    :output output-stream
+                                    :error error-stream
+                                    :status-hook status-hook
+                                    :cookie cookie
+                                    #-win32 :pty #-win32 pty-stream
+                                    :%status :running
+                                    :pid child
+                                    #+win32 :copiers #+win32 *handlers-installed*
+                                    #+win32 :handle #+win32 handle))
+                             (with-active-processes-lock ()
+                               (push proc *active-processes*))
+                             ;; In case a sigchld signal was missed
+                             ;; when the process wasn't on
+                             ;; *active-processes*
+                             (unless wait
+                               (get-processes-status-changes)))))))
+                   ;; Report the error outside the lock.
+                   (case child
+                     (-1
+                      (error "Couldn't fork child process: ~A"
+                             (strerror)))
+                     (-2
+                      (error "Couldn't execute ~S: ~A"
+                             progname (strerror)))
+                     (-3
+                      (error "Couldn't change directory to ~S: ~A"
+                             directory (strerror)))))))))
       (dolist (fd *close-in-parent*)
         (sb-unix:unix-close fd))
       (unless proc
@@ -942,10 +1010,9 @@ Users Manual for details about the PROCESS structure.
                                          (sap+ (vector-sap buf) read-end)
                                          (- (length buf) read-end)))
                   (cond
-                    ((and #-win32 (or (and (null count)
-                                           (eql errno sb-unix:eio))
-                                      (eql count 0))
-                          #+win32 (<= count 0))
+                    ((or (and (null count)
+                              (eql errno sb-unix:eio))
+                         (eql count 0))
                      (remove-fd-handler handler)
                      (setf handler nil)
                      (decf (car cookie))
@@ -988,7 +1055,7 @@ Users Manual for details about the PROCESS structure.
      (values (fd-stream-fd stream) nil (stream-external-format stream)))
     (synonym-stream
      (get-stream-fd-and-external-format
-      (symbol-value (synonym-stream-symbol stream)) direction))
+      (resolve-synonym-stream stream) direction))
     (two-way-stream
      (ecase direction
        (:input
@@ -1037,7 +1104,7 @@ Users Manual for details about the PROCESS structure.
                  (sb-unix:unix-close fd)
                  (fail "failed to unlink ~A" name/errno))
                fd)))
-    (let ((dev-null #.(coerce #-win32 "/dev/null" #+win32 "nul" 'base-string)))
+    (let ((dev-null #-win32 "/dev/null" #+win32 "nul"))
       (cond ((eq object t)
              ;; No new descriptor is needed.
              (values -1 nil))
@@ -1210,3 +1277,14 @@ Users Manual for details about the PROCESS structure.
                   (return (values write-fd nil)))))))
           (t
            (fail "invalid option: ~S" object))))))
+
+#+(or linux sunos hpux)
+(defun software-version ()
+  "Return a string describing version of the supporting software, or NIL
+  if not available."
+  (or sb-sys::*software-version*
+      (setf sb-sys::*software-version*
+            (string-trim '(#\newline)
+                         (sb-kernel:with-simple-output-to-string (stream)
+                           (run-program "/bin/uname" `("-r")
+                                        :output stream))))))
