@@ -10,9 +10,9 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
-#!-sb-fluid
+#-sb-fluid
 (declaim (inline clear-bit-vector set-bit-vector bit-vector-replace
                  bit-vector-copy))
 
