@@ -65,22 +65,17 @@
 ;;; have these meanings:
 ;;;     NIL     No declaration seen: do whatever you feel like, but don't
 ;;;             dump an inline expansion.
-;;; :NOTINLINE  NOTINLINE declaration seen: always do full function call.
-;;;    :INLINE  INLINE declaration seen: save expansion, expanding to it
+;;; NOTINLINE  NOTINLINE declaration seen: always do full function call.
+;;;    INLINE  INLINE declaration seen: save expansion, expanding to it
 ;;;             if policy favors.
-;;; :MAYBE-INLINE
+;;; MAYBE-INLINE
 ;;;             Retain expansion, but only use it opportunistically.
-;;;             :MAYBE-INLINE is quite different from :INLINE. As explained
+;;;             MAYBE-INLINE is quite different from INLINE. As explained
 ;;;             by APD on #lisp 2005-11-26: "MAYBE-INLINE lambda is
 ;;;             instantiated once per component, INLINE - for all
 ;;;             references (even under #'without FUNCALL)."
 (def!type inlinep ()
-  '(member :inline :maybe-inline :notinline nil))
-(defconstant-eqx +inlinep-translations+
-  '((inline . :inline)
-    (notinline . :notinline)
-    (maybe-inline . :maybe-inline))
-  #'equal)
+  '(member inline maybe-inline notinline nil))
 
 (defstruct (dxable-args (:constructor make-dxable-args (list))
                         (:predicate nil)
@@ -137,7 +132,6 @@
 (defvar *lambda-conversions*)
 (defvar *compile-object* nil)
 (defvar *location-context* nil)
-(defvar *msan-unpoison* nil)
 
 (defvar *stack-allocate-dynamic-extent* t
   "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
@@ -350,6 +344,38 @@ the stack without triggering overflow protection.")
     (list (make-hash-table :test 'equal :synchronized t)))
   (declaim (type (cons hash-table) *code-coverage-info*)))
 
+(defstruct (compilation (:copier nil)
+                        (:predicate nil)
+                        (:conc-name ""))
+  (fun-names-in-this-file)
+  (coverage-metadata nil :type (or (cons hash-table hash-table) null) :read-only t)
+  (msan-unpoison nil :read-only t)
+  (sset-counter 1 :type fixnum)
+  ;; Bidrectional map between IR1/IR2/assembler abstractions
+  ;; and a corresponding small integer identifier. One direction could be done
+  ;; by adding the integer ID as an object slot, but we want both directions.
+  ;; These could just as well be scoped by WITH-IR1-NAMESPACE, but
+  ;; since it's primarily a debugging tool, it's nicer to have
+  ;; a wider unique scope by ID.
+  (objmap-obj-to-id   (make-hash-table :test 'eq) :read-only t)
+  (objmap-id-to-cont  (make-array 10) :type simple-vector) ; number -> CTRAN or LVAR
+  (objmap-id-to-tn    (make-array 10) :type simple-vector) ; number -> TN
+  (objmap-id-to-label (make-array 10) :type simple-vector) ; number -> LABEL
+  (objmap-cont-num    0 :type fixnum)
+  (objmap-tn-id       0 :type fixnum)
+  (objmap-label-id    0 :type fixnum)
+  ;; if emitting a cfasl, the fasl stream to that
+  (compile-toplevel-object nil :read-only t)
+  ;; these are all historical baggage from here down,
+  ;; unused unless we ever decide to fix block compilation
+  (block-compile nil :type (member nil t :specified))
+  ;; When block compiling, used by PROCESS-FORM to accumulate top level
+  ;; lambdas resulting from compiling subforms. (In reverse order.)
+  (toplevel-lambdas nil :type list))
+
+(defvar *compilation*)
+(declaim (type compilation *compilation*))
+
 (in-package "SB-ALIEN")
 
 ;;; Information describing a heap-allocated alien.
