@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (defconstant arg-count-sc (make-sc+offset any-reg-sc-number rcx-offset))
 (defconstant closure-sc (make-sc+offset any-reg-sc-number rax-offset))
@@ -100,11 +100,8 @@
 
 (macrolet ((define-frame-op
                (suffix sc stack-sc instruction
-                &optional (ea
-                           `(make-ea :qword
-                                     :base frame-pointer
-                                     :disp (frame-byte-offset
-                                            (tn-offset variable-home-tn)))))
+                &optional (ea `(ea (frame-byte-offset (tn-offset variable-home-tn))
+                                   frame-pointer)))
                (let ((reffer (symbolicate 'ancestor-frame-ref '/ suffix))
                      (setter (symbolicate 'ancestor-frame-set '/ suffix)))
                  `(progn
@@ -141,10 +138,10 @@
                                 (list (primitive-type-or-lose ',name)
                                       (sc-or-lose ',stack-sc)
                                       (lambda (node block fp value res)
-                                        (sb!c::vop ,ref node block
+                                        (sb-c::vop ,ref node block
                                                    fp value res))
                                       (lambda (node block fp new-val value)
-                                        (sb!c::vop ,set node block
+                                        (sb-c::vop ,set node block
                                                    fp new-val value)))))))))
     (foo (double-float double-stack
                        ancestor-frame-ref/double-float
@@ -174,13 +171,14 @@
 (define-vop (xep-allocate-frame)
   (:info start-lab)
   (:generator 1
-    (emit-alignment n-lowtag-bits)
+    (let ((nop-kind
+           (shiftf (sb-assem::asmstream-inter-function-padding sb-assem:*asmstream*)
+                   :nop)))
+      (emit-alignment n-lowtag-bits (if (eq nop-kind :nop) #x90 0)))
     (emit-label start-lab)
     ;; Skip space for the function header.
     (inst simple-fun-header-word)
-    (dotimes (i (* n-word-bytes (1- simple-fun-code-offset)))
-      (inst byte 0))
-
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     ;; The start of the actual code.
     ;; Save the return-pc.
     (popw rbp-tn (frame-word-offset return-pc-save-offset))))
@@ -188,7 +186,7 @@
 (defun emit-lea (target source disp)
   (if (eql disp 0)
       (inst mov target source)
-      (inst lea target (make-ea :qword :base source :disp disp))))
+      (inst lea target (ea disp source))))
 
 (define-vop (xep-setup-sp)
   (:generator 1
@@ -205,8 +203,7 @@
   (:info callee)
   (:ignore nfp callee)
   (:generator 2
-    (inst lea res (make-ea :qword :base rsp-tn
-                           :disp (- (* sp->fp-offset n-word-bytes))))
+    (inst lea res (ea (- (* sp->fp-offset n-word-bytes)) rsp-tn))
     (inst sub rsp-tn (* n-word-bytes (sb-allocated-size 'stack)))))
 
 (defun make-stack-pointer-tn (&optional nargs)
@@ -217,7 +214,7 @@
               (* (max (if (> nargs register-arg-count)
                           nargs
                           0)
-                      (sb!c::sb-size (sb-or-lose 'stack)))
+                      (sb-c::sb-size (sb-or-lose 'stack)))
                  n-word-bytes)))
       (make-wired-tn *fixnum-primitive-type* any-reg-sc-number rsp-offset)
       (make-normal-tn *fixnum-primitive-type*)))
@@ -234,13 +231,13 @@
           (stack-size (* (max (if (> nargs register-arg-count)
                                   nargs
                                   0)
-                              (sb!c::sb-size (sb-or-lose 'stack)))
+                              (sb-c::sb-size (sb-or-lose 'stack)))
                          n-word-bytes)))
       (cond ((= fp-offset stack-size)
              (inst sub rsp-tn stack-size)
              (move res rsp-tn))
             (t
-             (inst lea res (make-ea :qword :base rsp-tn :disp (- fp-offset)))
+             (inst lea res (ea (- fp-offset) rsp-tn))
              (inst sub rsp-tn stack-size))))))
 
 ;;; Emit code needed at the return-point from an unknown-values call
@@ -264,12 +261,12 @@
 (defun default-unknown-values (vop values nvals node)
   (declare (type (or tn-ref null) values)
            (type unsigned-byte nvals))
-  (let ((type (sb!c::basic-combination-derived-type node)))
+  (let ((type (sb-c::basic-combination-derived-type node)))
     (cond
       ((<= nvals 1)
        (note-this-location vop :single-value-return)
        (cond
-         ((<= (sb!kernel:values-type-max-value-count type)
+         ((<= (sb-kernel:values-type-max-value-count type)
               register-arg-count)
           (when (and (named-type-p type)
                      (eq nil (named-type-name type)))
@@ -277,13 +274,13 @@
             ;; ends right here leavig the :SINGLE-VALUE-RETURN note
             ;; dangling. Let's emit a NOP.
             (inst nop)))
-         ((not (sb!kernel:values-type-may-be-single-value-p type))
+         ((not (sb-kernel:values-type-may-be-single-value-p type))
           (inst mov rsp-tn rbx-tn))
          (t
           (inst cmov :c rsp-tn rbx-tn))))
       ((<= nvals register-arg-count)
        (note-this-location vop :unknown-return)
-       (when (sb!kernel:values-type-may-be-single-value-p type)
+       (when (sb-kernel:values-type-may-be-single-value-p type)
          (let ((regs-defaulted (gen-label)))
            (inst jmp :c regs-defaulted)
            ;; Default the unsupplied registers.
@@ -299,14 +296,12 @@
                  for count from 2 below register-arg-count
                  unless (eq (tn-kind (tn-ref-tn tn-ref)) :unused)
                  do
-                 (inst mov (reg-in-size (tn-ref-tn tn-ref) :dword)
-                       (if 2nd-tn-live
-                           (reg-in-size 2nd-tn :dword)
-                           nil-value)))))
+                 (inst mov :dword (tn-ref-tn tn-ref)
+                       (if 2nd-tn-live 2nd-tn nil-value)))))
            (inst mov rbx-tn rsp-tn)
            (emit-label regs-defaulted)))
        (when (< register-arg-count
-                (sb!kernel:values-type-max-value-count type))
+                (sb-kernel:values-type-max-value-count type))
          (inst mov rsp-tn rbx-tn)))
       (t
        (collect ((defaults))
@@ -336,7 +331,9 @@
                 (unless (eq (tn-kind tn) :unused)
                   (let ((default-lab (gen-label)))
                     (defaults (cons default-lab tn))
-                    (inst cmp ecx-tn (fixnumize i))
+                    ;; Note that the max number of values received
+                    ;; is assumed to fit in a :dword register.
+                    (inst cmp :dword rcx-tn (fixnumize i))
                     (inst jmp :be default-lab)
                     (sc-case tn
                       (control-stack
@@ -348,7 +345,7 @@
             (move rsp-tn rbx-tn)
             (let ((defaults (defaults)))
               (when defaults
-                (assemble (*elsewhere*)
+                (assemble (:elsewhere)
                   (emit-label default-stack-slots)
                   (dolist (default defaults)
                     (emit-label (car default))
@@ -376,16 +373,16 @@
 ;;; them.)
 (defun receive-unknown-values (args nargs start count node)
   (declare (type tn args nargs start count))
-  (let ((type (sb!c::basic-combination-derived-type node))
+  (let ((type (sb-c::basic-combination-derived-type node))
         (variable-values (gen-label))
         (stack-values (gen-label))
         (done (gen-label))
         (unused-count-p (eq (tn-kind count) :unused)))
-    (when (sb!kernel:values-type-may-be-single-value-p type)
+    (when (sb-kernel:values-type-may-be-single-value-p type)
       (inst jmp :c variable-values)
       (cond ((location= start (first *register-arg-tns*))
              (inst push (first *register-arg-tns*))
-             (inst lea start (make-ea :qword :base rsp-tn :disp n-word-bytes)))
+             (inst lea start (ea n-word-bytes rsp-tn)))
             (t (inst mov start rsp-tn)
                (inst push (first *register-arg-tns*))))
       (unless (eq (tn-kind count) :unused)
@@ -394,13 +391,13 @@
       (emit-label variable-values))
     ;; The stack frame is burnt and RETurned from if there are no
     ;; stack values. In this case quickly reallocate sufficient space.
-    (when (<= (sb!kernel:values-type-min-value-count type)
+    (when (<= (sb-kernel:values-type-min-value-count type)
               register-arg-count)
       (inst cmp nargs (fixnumize register-arg-count))
       (inst jmp :g stack-values)
-      #!+#.(cl:if (cl:= sb!vm:word-shift sb!vm:n-fixnum-tag-bits) '(and) '(or))
+      #+#.(cl:if (cl:= sb-vm:word-shift sb-vm:n-fixnum-tag-bits) '(and) '(or))
       (inst sub rsp-tn nargs)
-      #!-#.(cl:if (cl:= sb!vm:word-shift sb!vm:n-fixnum-tag-bits) '(and) '(or))
+      #-#.(cl:if (cl:= sb-vm:word-shift sb-vm:n-fixnum-tag-bits) '(and) '(or))
       (progn
         ;; FIXME: This can't be efficient, but LEA (my first choice)
         ;; doesn't do subtraction.
@@ -415,7 +412,7 @@
     (loop
       for arg in *register-arg-tns*
       for i downfrom -1
-      for j below (sb!kernel:values-type-max-value-count type)
+      for j below (sb-kernel:values-type-max-value-count type)
       do (storew arg args i))
     (move start args)
     (unless unused-count-p
@@ -441,13 +438,13 @@
 (defun check-ocfp-and-return-pc (old-fp return-pc)
   #+nil
   (format t "*known-return: old-fp ~S, tn-kind ~S; ~S ~S~%"
-          old-fp (tn-kind old-fp) (sb!c::tn-save-tn old-fp)
-          (tn-kind (sb!c::tn-save-tn old-fp)))
+          old-fp (tn-kind old-fp) (sb-c::tn-save-tn old-fp)
+          (tn-kind (sb-c::tn-save-tn old-fp)))
   #+nil
   (format t "*known-return: return-pc ~S, tn-kind ~S; ~S ~S~%"
           return-pc (tn-kind return-pc)
-          (sb!c::tn-save-tn return-pc)
-          (tn-kind (sb!c::tn-save-tn return-pc)))
+          (sb-c::tn-save-tn return-pc)
+          (tn-kind (sb-c::tn-save-tn return-pc)))
   (unless (and (sc-is old-fp control-stack)
                (= (tn-offset old-fp) ocfp-save-offset))
     (error "ocfp not on stack in standard save location?"))
@@ -607,7 +604,7 @@
 ;;; have been set up in the current frame.
 (macrolet ((define-full-call (vop-name named return variable)
             (aver (not (and variable (eq return :tail))))
-            #!+immobile-code (when named (setq named :direct))
+            #+immobile-code (when named (setq named :direct))
             `(define-vop (,vop-name ,@(when (eq return :unknown)
                                         '(unknown-values-receiver)))
                (:args
@@ -648,7 +645,7 @@
                ,@(when (eq return :fixed) '(nvals))
                step-instrumenting
                ,@(unless named
-                   '(callable)))
+                   '(fun-type)))
 
                (:ignore
                ,@(unless (or variable (eq return :tail)) '(arg-locs))
@@ -800,7 +797,7 @@
 
                (when (and step-instrumenting
                           ,@(and (eq named :direct)
-                                 `((not (and #!+immobile-code
+                                 `((not (and #+immobile-code
                                              ;; handle-single-step-around-trap can't handle it
                                              (static-fdefn-offset fun))))))
                  (emit-single-step-test)
@@ -809,14 +806,14 @@
                DONE
                (note-this-location vop :call-site)
                ,(cond ((eq named :direct)
-                       #!+immobile-code
+                       #+immobile-code
                        `(let* ((fixup (make-fixup
                                        fun
                                        (if (static-fdefn-offset fun)
                                            :static-call
                                            :named-call)))
                                (target
-                                 (if (and (sb!c::code-immobile-p node)
+                                 (if (and (sb-c::code-immobile-p node)
                                           (not step-instrumenting))
                                      fixup
                                      (progn
@@ -829,21 +826,18 @@
                                        (inst mov rax-tn fixup)
                                        rax-tn))))
                           (inst ,(if (eq return :tail) 'jmp 'call) target))
-                       #!-immobile-code
+                       #-immobile-code
                        `(inst ,(if (eq return :tail) 'jmp 'call)
-                              (make-ea :qword :disp
-                                       (+ nil-value (static-fun-offset fun)))))
-                      #!-immobile-code
+                              (ea (+ nil-value (static-fun-offset fun)))))
+                      #-immobile-code
                       (named
                        `(inst ,(if (eq return :tail) 'jmp 'call)
-                              (make-ea :qword :base rax
-                                              :disp (- (* fdefn-raw-addr-slot
-                                                          n-word-bytes)
-                                                       other-pointer-lowtag))))
+                              (ea (- (* fdefn-raw-addr-slot n-word-bytes)
+                                     other-pointer-lowtag) rax)))
                       ((eq return :tail)
-                       `(tail-call-unnamed rax callable vop))
+                       `(tail-call-unnamed rax fun-type vop))
                       (t
-                       `(call-unnamed rax callable vop)))
+                       `(call-unnamed rax fun-type vop)))
                ,@(ecase return
                    (:fixed
                     '((default-unknown-values vop values nvals node)))
@@ -855,72 +849,58 @@
 
   (define-full-call call nil :fixed nil)
   (define-full-call call-named t :fixed nil)
-  #!-immobile-code
+  #-immobile-code
   (define-full-call static-call-named :direct :fixed nil)
   (define-full-call multiple-call nil :unknown nil)
   (define-full-call multiple-call-named t :unknown nil)
-  #!-immobile-code
+  #-immobile-code
   (define-full-call static-multiple-call-named :direct :unknown nil)
   (define-full-call tail-call nil :tail nil)
   (define-full-call tail-call-named t :tail nil)
-  #!-immobile-code
+  #-immobile-code
   (define-full-call static-tail-call-named :direct :tail nil)
 
   (define-full-call call-variable nil :fixed t)
   (define-full-call multiple-call-variable nil :unknown t))
 
-;;; Invoke the function-designator FUN. If DESIGNATOR-P is true, FUN might be
-;;; a symbol; if DESIGNATOR-P is false, FUN is definitely a function.
-;;;
-;;; Note: DESIGNATOR-P used to be named CALLABLE. They're both imperfect.
-;;; While it's true that the noun CALLABLE is synonymous with function-designator,
-;;; the former had a strange connotation of (NOT CALLABLE) counterintuitively
-;;; implying that the FUN *is* directly callable, being strictly a function,
-;;; with "callable" parsed in its adjectival sense.
-;;; Also problematic would be NOT-FUNCTIONP since FUN might satisfy FUNCTIONP.
-;;; A pedantically correct change would be to invert the boolean as supplied,
-;;; and name it DEFINITELY-FUNCTIONP, or name it just FUNCTIONP with the
-;;; notion that T means T and NIL means possibly.
-;;; However, we might really want three values: yes, no, and maybe.
-;;; Because as things are, the definitely-NOT-function case goes through
-;;; the "maybe" path even though it _must_ coerce to a function:
-;;;
-;;; * (disassemble '(lambda (f) (funcall (the symbol f) 42)))
-;;;  3F:       8D58F5           LEA EBX, [RAX-11]
-;;;  42:       F6C30F           TEST BL, 15
-;;;  45:       7503             JNE L1
-;;;  47:       FF60FD           JMP QWORD PTR [RAX-3]
-;;;  4A: L1:   FF242570000020   JMP QWORD PTR [#x20000070]       ; TAIL-CALL-SYMBOL
+;;; Invoke the function-designator FUN.
+(defun tail-call-unnamed (fun type vop)
+  (let ((relative-call (sb-c::code-immobile-p vop))
+        (fun-ea (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag)
+                    fun)))
+    (case type
+      (:designator
+       (assemble ()
+         (%lea-for-lowtag-test rbx-tn fun fun-pointer-lowtag)
+         (inst test :byte rbx-tn lowtag-mask)
+         (inst jmp :nz (if relative-call
+                           (make-fixup 'call-symbol :assembly-routine)
+                           not-fun))
+         (inst jmp fun-ea)
+         not-fun
+         (unless relative-call
+           (invoke-asm-routine 'jmp 'call-symbol vop))))
+      (:symbol
+       (invoke-asm-routine 'jmp 'call-symbol vop))
+      (t
+       (inst jmp fun-ea)))))
 
-(defun tail-call-unnamed (fun designator-p vop)
-  (let ((relative-call (sb!c::code-immobile-p vop))
-        (fun-ea (make-ea :qword :base fun
-                         :disp (- (* closure-fun-slot n-word-bytes)
-                                  fun-pointer-lowtag))))
-    (if designator-p
-        (assemble ()
-          (%lea-for-lowtag-test ebx-tn fun fun-pointer-lowtag)
-          (inst test bl-tn lowtag-mask)
-          (inst jmp :nz (if relative-call
-                            (make-fixup 'tail-call-symbol :assembly-routine)
-                            not-fun))
-          (inst jmp fun-ea)
-          not-fun
-          (unless relative-call
-            (invoke-asm-routine 'jmp 'tail-call-symbol vop)))
-        (inst jmp fun-ea))))
-
-(defun call-unnamed (fun designator-p vop)
-  (assemble ()
-    (when designator-p
-      (%lea-for-lowtag-test ebx-tn fun fun-pointer-lowtag)
-      (inst test bl-tn lowtag-mask)
-      (inst jmp :z call)
-      (invoke-asm-routine 'call 'call-symbol vop))
-    call
-    (inst call (make-ea :qword :base fun
-                        :disp (- (* closure-fun-slot n-word-bytes)
-                                 fun-pointer-lowtag)))))
+(defun call-unnamed (fun type vop)
+  (case type
+    (:symbol
+     (invoke-asm-routine 'call 'call-symbol vop))
+    (t
+     (assemble ()
+       (when (eq type :designator)
+         (%lea-for-lowtag-test rbx-tn fun fun-pointer-lowtag)
+         (inst test :byte rbx-tn lowtag-mask)
+         (inst jmp :z call)
+         (invoke-asm-routine 'call 'call-symbol vop)
+         (inst jmp ret))
+       call
+       (inst call (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag)
+                      fun))
+       ret))))
 
 ;;; This is defined separately, since it needs special code that BLT's
 ;;; the arguments down. All the real work is done in the assembly
@@ -930,7 +910,7 @@
          (function :scs (descriptor-reg control-stack) :target rax)
          (old-fp)
          (return-pc))
-  (:info callable)
+  (:info fun-type)
   (:temporary (:sc unsigned-reg :offset rsi-offset :from (:argument 0)) rsi)
   (:temporary (:sc unsigned-reg :offset rax-offset :from (:argument 1)) rax)
   (:vop-var vop)
@@ -940,9 +920,9 @@
     (move rsi args)
     (move rax function)
     ;; And jump to the assembly routine.
-    (invoke-asm-routine 'jmp (if callable
-                                 'tail-call-callable-variable
-                                 'tail-call-variable)
+    (invoke-asm-routine 'jmp (if (eq fun-type :function)
+                                 'tail-call-variable
+                                 'tail-call-callable-variable)
                         vop)))
 
 ;;;; unknown values return
@@ -1005,8 +985,7 @@
       ;; This is handled in RETURN-SINGLE.
       (error "nvalues is 1"))
     ;; Establish the values pointer and values count.
-    (inst lea rbx (make-ea :qword :base rbp-tn
-                           :disp (* sp->fp-offset n-word-bytes)))
+    (inst lea rbx (ea (* sp->fp-offset n-word-bytes) rbp-tn))
     (if (zerop nvals)
         (zeroize rcx) ; smaller
         (inst mov rcx (fixnumize nvals)))
@@ -1034,13 +1013,11 @@
            ;; Clear as much of the stack as possible, but not past the
            ;; old frame address.
            (inst lea rsp-tn
-                 (make-ea :qword :base rbp-tn
-                          :disp (frame-byte-offset (1- nvals))))
+                 (ea (frame-byte-offset (1- nvals)) rbp-tn))
            (move rbp-tn old-fp)
-           (inst push (make-ea :qword :base rbx
-                               :disp (frame-byte-offset
-                                      (+ sp->fp-offset
-                                         (tn-offset return-pc)))))
+           (inst push (ea (frame-byte-offset
+                           (+ sp->fp-offset (tn-offset return-pc)))
+                          rbx))
            (inst ret)))))
 
 ;;; Do unknown-values return of an arbitrary number of values (passed
@@ -1129,11 +1106,8 @@
     (inst lea (if (<= fixed (sb-allocated-size 'stack))
                   rsp-tn
                   source)
-          (make-ea :qword :base rbp-tn
-                          :index temp :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                          :disp (* n-word-bytes
-                                   (- (+ sp->fp-offset fixed)
-                                      (sb-allocated-size 'stack)))))
+          (ea (* n-word-bytes (- (+ sp->fp-offset fixed) (sb-allocated-size 'stack)))
+              rbp-tn temp (ash 1 (- word-shift n-fixnum-tag-bits))))
 
     ;; Now: nargs>=1 && nargs>fixed
 
@@ -1154,17 +1128,15 @@
            (inst jmp :be DO-REGS))
           (t
            ;; Number to copy = nargs-fixed
-           (inst lea rbx-tn (make-ea :qword :base rcx-tn
-                                     :disp (- (fixnumize fixed))))))
+           (inst lea rbx-tn (ea (- (fixnumize fixed)) rcx-tn))))
 
     ;; Initialize R8 to be the end of args.
     ;; Swap with SP if necessary to mirror the previous condition
     (inst lea (if (<= fixed (sb-allocated-size 'stack))
                   source
                   rsp-tn)
-          (make-ea :qword :base rbp-tn
-                          :index temp :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                          :disp (* sp->fp-offset n-word-bytes)))
+          (ea (* sp->fp-offset n-word-bytes)
+              rbp-tn temp (ash 1 (- word-shift n-fixnum-tag-bits))))
 
     ;; src: rbp + temp + sp->fp
     ;; dst: rbp + temp + sp->fp + (fixed - [stack-size])
@@ -1179,8 +1151,8 @@
              ;; much worse than an explicit loop for small blocks.
 
              (emit-label loop)
-             (inst mov temp (make-ea :qword :base source :index copy-index))
-             (inst mov (make-ea :qword :base rsp-tn :index copy-index) temp)
+             (inst mov temp (ea source copy-index))
+             (inst mov (ea rsp-tn copy-index) temp)
              (inst add copy-index n-word-bytes)
              (inst sub rbx-tn (fixnumize 1))
              (inst jmp :nz loop))
@@ -1188,11 +1160,8 @@
              ;; dst is higher than src; copy backward
              (emit-label loop)
              (inst sub rbx-tn (fixnumize 1))
-             (inst mov temp (make-ea :qword :base rsp-tn
-                                     :index rbx-tn :scale fixnum->word))
-             (inst mov (make-ea :qword :base source
-                                :index rbx-tn :scale fixnum->word)
-                   temp)
+             (inst mov temp (ea rsp-tn rbx-tn fixnum->word))
+             (inst mov (ea source rbx-tn fixnum->word) temp)
              (inst jmp :nz loop)
              ;; done with the stack--stack copy. Reset RSP to its final
              ;; value
@@ -1206,12 +1175,10 @@
       (do ((i fixed))
           ( nil )
         ;; Store it relative to rbp
-        (inst mov (make-ea :qword :base rbp-tn
-                           :disp (* n-word-bytes
-                                    (- sp->fp-offset
-                                       (+ 1
-                                          (- i fixed)
-                                          (sb-allocated-size 'stack)))))
+        (inst mov (ea (* n-word-bytes
+                         (- sp->fp-offset
+                            (+ 1 (- i fixed) (sb-allocated-size 'stack))))
+                       rbp-tn)
               (nth i *register-arg-tns*))
 
         (incf i)
@@ -1219,9 +1186,7 @@
           (return))
 
         ;; Don't deposit any more than there are.
-        (if (zerop i)
-            (inst test rcx-tn rcx-tn)
-            (inst cmp rcx-tn (fixnumize i)))
+        (inst cmp :dword rcx-tn (fixnumize i))
         (inst jmp :eq DONE)))
 
     (inst jmp DONE)
@@ -1235,7 +1200,7 @@
     DONE))
 
 (define-vop (more-kw-arg)
-  (:translate sb!c::%more-kw-arg)
+  (:translate sb-c::%more-kw-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1))
          (index :scs (any-reg) :to (:result 1) :target keyword))
@@ -1244,26 +1209,23 @@
             (keyword :scs (descriptor-reg any-reg)))
   (:result-types * *)
   (:generator 4
-     (inst mov value (make-ea :qword :base object :index index
-                              :scale (ash 1 (- word-shift n-fixnum-tag-bits))))
-     (inst mov keyword (make-ea :qword :base object :index index
-                                :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                                :disp n-word-bytes))))
+     (inst mov value (ea object index (ash 1 (- word-shift n-fixnum-tag-bits))))
+     (inst mov keyword (ea n-word-bytes object index
+                           (ash 1 (- word-shift n-fixnum-tag-bits))))))
 
 (define-vop (more-arg/c)
-  (:translate sb!c::%more-arg)
+  (:translate sb-c::%more-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1)))
   (:info index)
-  (:arg-types * (:constant (signed-byte 32)))
+  (:arg-types * (:constant (signed-byte #.(- 32 word-shift))))
   (:results (value :scs (descriptor-reg any-reg)))
   (:result-types *)
   (:generator 3
-    (inst mov value (make-ea :qword :base object
-                                    :disp (- (* index n-word-bytes))))))
+    (inst mov value (ea (- (* index n-word-bytes)) object))))
 
 (define-vop (more-arg)
-  (:translate sb!c::%more-arg)
+  (:translate sb-c::%more-arg)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg) :to (:result 1))
          (index :scs (any-reg) :to (:result 1) :target value))
@@ -1273,8 +1235,8 @@
   (:generator 4
     (move value index)
     (inst neg value)
-    (inst mov value (make-ea :qword :base object :index value
-                             :scale (ash 1 (- word-shift n-fixnum-tag-bits))))))
+    (inst mov value (ea object value
+                        (ash 1 (- word-shift n-fixnum-tag-bits))))))
 
 ;;; Turn more arg (context, count) into a list.
 (define-vop (listify-rest-args)
@@ -1299,8 +1261,10 @@
       ;; Check to see whether there are no args, and just return NIL if so.
       (inst mov result nil-value)
       (inst jrcxz done)
-      (inst lea dst (make-ea :qword :index rcx :scale (ash 2 (- word-shift n-fixnum-tag-bits))))
-      (maybe-pseudo-atomic stack-allocate-p
+      (inst lea dst (ea nil rcx (ash 2 (- word-shift n-fixnum-tag-bits))))
+      (unless stack-allocate-p
+        (instrument-alloc dst node))
+      (pseudo-atomic (:elide-if stack-allocate-p)
        ;; FIXME: if COUNT >= 8192, allocates to single-object page(s).
        ;; All we have to do is unset the '.singleton' bit,
        ;; a permissible state change as long as pseudo-atomic.
@@ -1317,7 +1281,7 @@
        (storew dst dst -1 list-pointer-lowtag)
        (emit-label enter)
        ;; Grab one value and stash it in the car of this cons.
-       (inst mov rax (make-ea :qword :base src))
+       (inst mov rax (ea src))
        (inst sub src n-word-bytes)
        (storew rax dst 0 list-pointer-lowtag)
        ;; Go back for more.
@@ -1339,7 +1303,7 @@
 ;;; below the current stack top.
 (define-vop (more-arg-context)
   (:policy :fast-safe)
-  (:translate sb!c::%more-arg-context)
+  (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg) :target count))
   (:arg-types positive-fixnum (:constant fixnum))
   (:info fixed)
@@ -1351,10 +1315,9 @@
     (move count supplied)
     ;; SP at this point points at the last arg pushed.
     ;; Point to the first more-arg, not above it.
-    (inst lea context (make-ea :qword :base rsp-tn
-                               :index count
-                               :scale (ash 1 (- word-shift n-fixnum-tag-bits))
-                               :disp (- (* (1+ fixed) n-word-bytes))))
+    (inst lea context (ea (- (* (1+ fixed) n-word-bytes))
+                          rsp-tn count
+                          (ash 1 (- word-shift n-fixnum-tag-bits))))
     (unless (zerop fixed)
       (inst sub count (fixnumize fixed)))))
 
@@ -1398,7 +1361,7 @@
 ;; in regard to SB-C::%TYPE-CHECK-ERROR. Figure out why.
 (define-vop (type-check-error/word)
   (:policy :fast-safe)
-  (:translate sb!c::%type-check-error)
+  (:translate sb-c::%type-check-error)
   (:args (object :scs (signed-reg unsigned-reg))
          ;; Types are trees of symbols, so 'any-reg' is not
          ;; really possible.
@@ -1413,7 +1376,7 @@
     (error-call vop 'object-not-type-error object type)))
 (define-vop (type-check-error/word/c)
   (:policy :fast-safe)
-  (:translate sb!c::%type-check-error/c)
+  (:translate sb-c::%type-check-error/c)
   (:args (object :scs (signed-reg unsigned-reg)))
   (:arg-types untagged-num (:constant symbol) (:constant t))
   (:info errcode *location-context*)
@@ -1433,15 +1396,8 @@
   ;; and reading a slot from a thread structure would require an extra
   ;; register on -SB-THREAD. While this isn't critical for x86-64,
   ;; it's more serious for x86.
-  #!+sb-thread
-  (inst cmp (thread-tls-ea (* thread-stepping-slot n-word-bytes)) 0)
-  #!-sb-thread
-  (inst cmp (make-ea :byte
-                     :disp (+ nil-value (static-symbol-offset
-                                         'sb!impl::*stepping*)
-                              (* symbol-value-slot n-word-bytes)
-                              (- other-pointer-lowtag)))
-        0))
+  #+sb-thread (inst cmp :byte (thread-slot-ea thread-stepping-slot) 0)
+  #-sb-thread (inst cmp :byte (static-symbol-value-ea 'sb-impl::*stepping*) 0))
 
 (define-vop (step-instrument-before-vop)
   (:policy :fast-safe)
@@ -1451,4 +1407,4 @@
      (inst jmp :eq DONE)
      (inst break single-step-before-trap)
      DONE
-     (note-this-location vop :step-before-vop)))
+     (note-this-location vop :internal-error)))
