@@ -15,8 +15,12 @@
 #include "gencgc-private.h"
 #include "genesis/gc-tables.h"
 #include "genesis/closure.h"
+#include "genesis/cons.h"
+#include "genesis/instance.h"
+#include "genesis/vector.h"
 #include "genesis/layout.h"
 #include "genesis/hash-table.h"
+#include "code.h"
 #include "immobile-space.h"
 #include "queue.h"
 
@@ -167,9 +171,9 @@ static inline int pointer_survived_gc_yet(lispobj pointer)
     if (listp(pointer))
         return cons_markedp(pointer);
     lispobj header = *native_pointer(pointer);
-    if (widetag_of(header) == BIGNUM_WIDETAG)
+    if (header_widetag(header) == BIGNUM_WIDETAG)
         return (header & BIGNUM_MARK_BIT) != 0;
-    if (embedded_obj_p(widetag_of(header)))
+    if (embedded_obj_p(header_widetag(header)))
         header = *fun_code_header(native_pointer(pointer));
     return (header & MARK_BIT) != 0;
 }
@@ -182,11 +186,11 @@ void __mark_obj(lispobj pointer)
     if (!listp(pointer)) {
         lispobj* base = native_pointer(pointer);
         lispobj header = *base;
-        if (widetag_of(header) == BIGNUM_WIDETAG) {
+        if (header_widetag(header) == BIGNUM_WIDETAG) {
             *base |= BIGNUM_MARK_BIT;
             return; // don't enqueue - no pointers
         } else {
-            if (embedded_obj_p(widetag_of(header))) {
+            if (embedded_obj_p(header_widetag(header))) {
                 base = fun_code_header(base);
                 pointer = make_lispobj(base, OTHER_POINTER_LOWTAG);
                 header = *base;
@@ -200,7 +204,7 @@ void __mark_obj(lispobj pointer)
                 return; // already marked
             *base |= MARK_BIT;
         }
-        if (unboxed_obj_widetag_p(widetag_of(header)))
+        if (leaf_obj_widetag_p(header_widetag(header)))
             return;
     } else {
         uword_t key = compute_page_key(pointer);
@@ -241,7 +245,7 @@ void gc_mark_range(lispobj* where, long count) {
 static void trace_object(lispobj* where)
 {
     lispobj header = *where;
-    int widetag = widetag_of(header);
+    int widetag = header_widetag(header);
     sword_t scan_from = 1;
     sword_t scan_to = sizetab[widetag](where);
     sword_t i;
@@ -267,7 +271,16 @@ static void trace_object(lispobj* where)
         // mixed boxed/unboxed objects
         bitmap = ((struct layout*)native_pointer(layout))->bitmap;
         // If no raw slots, just scan without use of the bitmap.
+        // A bitmap of -1 implies that not only are all slots tagged,
+        // there is no special GC method for any slot.
         if (bitmap == make_fixnum(-1)) break;
+        // Otherwise, the first slot might merit special treatment.
+        if (*where & CUSTOM_GC_SCAVENGE_FLAG) {
+            struct instance* node = (struct instance*)where;
+            lispobj next = node->slots[INSTANCE_DATA_START];
+            if (fixnump(next) && next) // ignore initially 0 heap words
+                __mark_obj(next|INSTANCE_POINTER_LOWTAG);
+        }
         for(i=1; i<scan_to; ++i)
             if (layout_bitmap_logbitp(i-1, bitmap) && is_lisp_pointer(where[i]))
                 __mark_obj(where[i]);
@@ -298,6 +311,10 @@ static void trace_object(lispobj* where)
             }
             return;
         }
+        if (is_vector_subtype(header, VectorWeak)) {
+            add_to_weak_vector_list(where, header);
+            return;
+        }
         break;
 #if defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64)
     /* on x86[-64], closure->fun is a fixnum-qua-pointer. Convert it to a lisp
@@ -308,11 +325,7 @@ static void trace_object(lispobj* where)
         break; // scan slots normally
 #endif
     case CODE_HEADER_WIDETAG:
-        for_each_simple_fun(i, fun, (struct code*)where, 0, {
-            gc_mark_range(SIMPLE_FUN_SCAV_START(fun),
-                          SIMPLE_FUN_SCAV_NWORDS(fun));
-        })
-        scan_to = code_header_words(header);
+        scan_to = code_header_words((struct code*)where);
         break;
     case FDEFN_WIDETAG:
         gc_mark_obj(fdefn_callee_lispobj((struct fdefn*)where));
@@ -321,10 +334,10 @@ static void trace_object(lispobj* where)
     case WEAK_POINTER_WIDETAG:
         weakptr = (struct weak_pointer*)where;
         if (is_lisp_pointer(weakptr->value) && interesting_pointer_p(weakptr->value))
-            add_to_weak_pointer_list(weakptr);
+            add_to_weak_pointer_chain(weakptr);
         return;
     default:
-        if (unboxed_obj_widetag_p(widetag)) return;
+        if (leaf_obj_widetag_p(widetag)) return;
     }
     for(i=scan_from; i<scan_to; ++i)
         gc_mark_obj(where[i]);
@@ -363,7 +376,7 @@ void execute_full_mark_phase()
     while (where < end) {
         lispobj obj = compute_lispobj(where);
         gc_enqueue(obj);
-        where += listp(obj) ? 2 : sizetab[widetag_of(*where)](where);
+        where += listp(obj) ? 2 : sizetab[widetag_of(where)](where);
     }
     do {
         lispobj ptr = gc_dequeue();
@@ -375,7 +388,7 @@ void execute_full_mark_phase()
     } while (scav_queue.head_block->count ||
              (test_weak_triggers(pointer_survived_gc_yet, gc_mark_obj) &&
               scav_queue.head_block->count));
-    gc_dispose_private_pages();
+
 #if HAVE_GETRUSAGE
     getrusage(RUSAGE_SELF, &after);
 #define timediff(b,a,field) \
@@ -389,22 +402,35 @@ void execute_full_mark_phase()
 #endif
 }
 
-static void smash_weak_pointers()
+static void local_smash_weak_pointers()
 {
     struct weak_pointer *wp, *next_wp;
-    for (wp = weak_pointers, next_wp = NULL; wp != NULL; wp = next_wp) {
-        gc_assert(widetag_of(wp->header)==WEAK_POINTER_WIDETAG);
-
+    for (wp = weak_pointer_chain; wp != WEAK_POINTER_CHAIN_END; wp = next_wp) {
+        gc_assert(widetag_of(&wp->header) == WEAK_POINTER_WIDETAG);
         next_wp = wp->next;
         wp->next = NULL;
-        if (next_wp == wp) /* gencgc uses a ref to self for end of list */
-            next_wp = NULL;
-
         lispobj pointee = wp->value;
         gc_assert(is_lisp_pointer(pointee));
         if (!pointer_survived_gc_yet(pointee))
             wp->value = UNBOUND_MARKER_WIDETAG;
     }
+    weak_pointer_chain = WEAK_POINTER_CHAIN_END;
+
+    struct cons* vectors = weak_vectors;
+    while (vectors) {
+        struct vector* vector = (struct vector*)vectors->car;
+        vectors = (struct cons*)vectors->cdr;
+        UNSET_WEAK_VECTOR_VISITED(vector);
+        sword_t len = fixnum_value(vector->length);
+        sword_t i;
+        for (i = 0; i<len; ++i) {
+            lispobj obj = vector->data[i];
+            // Ignore non-pointers
+            if (is_lisp_pointer(obj) && !pointer_survived_gc_yet(obj))
+                vector->data[i] = NIL;
+        }
+    }
+    weak_vectors = 0;
 }
 
 __attribute__((unused)) static char *fillerp(lispobj* where)
@@ -435,7 +461,7 @@ static void sweep_fixedobj_pages(long *zeroed)
 {
     low_page_index_t page;
 
-    for (page = 0 ; ; ++page) {
+    for (page = FIXEDOBJ_RESERVED_PAGES ; ; ++page) {
         lispobj *obj = fixedobj_page_address(page);
         if (obj >= fixedobj_free_pointer)
             break;
@@ -478,9 +504,9 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
                 }
             }
         } else {
-            nwords = sizetab[widetag_of(header)](where);
+            nwords = sizetab[header_widetag(header)](where);
             lispobj markbit =
-              widetag_of(header) != BIGNUM_WIDETAG ? MARK_BIT : BIGNUM_MARK_BIT;
+              header_widetag(header) != BIGNUM_WIDETAG ? MARK_BIT : BIGNUM_MARK_BIT;
             if (header & markbit)
                 *where = header ^ markbit;
             else {
@@ -490,14 +516,14 @@ static uword_t sweep(lispobj* where, lispobj* end, uword_t arg)
                     goto cons;
                 struct code* code  = (struct code*)where;
                 // Keep in sync with the definition of filler_obj_p()
-                lispobj header = CODE_HEADER_WIDETAG;
-                if (code->header != header) {
+                if (!filler_obj_p((lispobj*)code)) {
                     page_index_t page = find_page_index(where);
                     int gen = page >= 0 ? page_table[page].gen
                       : __immobile_obj_gen_bits(where);
                     NOTE_GARBAGE(gen, where, nwords, zeroed, {
-                        code->header = header;
-                        code->code_size = make_fixnum(nwords * N_WORD_BYTES);
+                        code->boxed_size = 0;
+                        code->header = (nwords << CODE_HEADER_SIZE_SHIFT)
+                                     | CODE_HEADER_WIDETAG;
                         memset(where+2, 0, (nwords - 2) * N_WORD_BYTES);
                     })
                 }
@@ -529,8 +555,9 @@ void execute_full_sweep_phase()
 {
     long words_zeroed[1+PSEUDO_STATIC_GENERATION]; // One count per generation
 
+    local_smash_weak_pointers();
+    gc_dispose_private_pages();
     cull_weak_hash_tables(alivep_funs);
-    smash_weak_pointers();
 
     memset(words_zeroed, 0, sizeof words_zeroed);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
