@@ -27,38 +27,11 @@
 #include "genesis/simple-fun.h"
 #include "thread.h"
 #include "interr.h" /* for lose() */
-
+#include "gc-assert.h"
+#include "code.h"
 extern const char *widetag_names[];
 extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 
-/// Enable extra debug-only checks if DEBUG
-#ifdef DEBUG
-# define gc_dcheck(ex) gc_assert(ex)
-#else
-# define gc_dcheck(ex) ((void)0)
-#endif
-
-/// Disable all assertions if NDEBUG
-#ifdef NDEBUG
-# define gc_assert(ex) ((void)0)
-# define gc_assert_verbose(ex, fmt, ...) ((void)0)
-#else
-# define gc_assert(ex)                                                 \
-do {                                                                   \
-    if (!(ex)) gc_abort();                                             \
-} while (0)
-# define gc_assert_verbose(ex, fmt, ...)                               \
-do {                                                                   \
-    if (!(ex)) {                                                       \
-        fprintf(stderr, fmt, ## __VA_ARGS__);                          \
-        gc_abort();                                                    \
-    }                                                                  \
-} while (0)
-#endif
-
-#define gc_abort()                                                     \
-  lose("GC invariant lost, file \"%s\", line %d\n", __FILE__, __LINE__)
-
 #ifdef LISP_FEATURE_GENCGC
 #include "gencgc-internal.h"
 #else
@@ -69,10 +42,15 @@ do {                                                                   \
 
 // Offset from an fdefn raw address to the underlying simple-fun,
 // if and only if it points to a simple-fun.
-#if defined(LISP_FEATURE_SPARC) || defined(LISP_FEATURE_ARM)
+// For those of us who are too memory-impaired to know how to use the value:
+//  - it is the amount to ADD to a tagged simple-fun pointer to get its entry address
+//  - or the amount to SUBTRACT from an entry address to get a tagged fun pointer
+// I almost might prefer two accessors named tagged_fun_to_fun_entry() and
+// and fun_entry_to_tagged_fun() instead of the manifest constant.
+#if defined(LISP_FEATURE_SPARC) || defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_RISCV)
 #define FUN_RAW_ADDR_OFFSET 0
 #else
-#define FUN_RAW_ADDR_OFFSET (offsetof(struct simple_fun, code) - FUN_POINTER_LOWTAG)
+#define FUN_RAW_ADDR_OFFSET (offsetof(struct simple_fun, insts) - FUN_POINTER_LOWTAG)
 #endif
 
 // For x86[-64], a simple-fun or closure's "self" slot is a fixum
@@ -96,9 +74,6 @@ do {                                                                   \
      | SIMPLE_VECTOR_WIDETAG)); \
   v->header ^= subtype_VectorWeakVisited << N_WIDETAG_BITS
 
-#define SIMPLE_FUN_SCAV_START(fun_ptr) &fun_ptr->name
-#define SIMPLE_FUN_SCAV_NWORDS(fun_ptr) ((lispobj*)fun_ptr->code - &fun_ptr->name)
-
 /* values for the *_alloc_* parameters, also see the commentary for
  * struct page in gencgc-internal.h. These constants are used in gc-common,
  * so they can't easily be made gencgc-only */
@@ -145,6 +120,9 @@ instance_scan(void (*proc)(lispobj*, sword_t, uword_t),
               lispobj *instance_ptr, sword_t n_words,
               lispobj bitmap, uword_t arg);
 
+extern int simple_fun_index(struct code*, struct simple_fun*);
+extern lispobj simple_fun_name(struct simple_fun*);
+
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
 static inline lispobj funinstance_layout(lispobj* funinstance_ptr) { // native ptr
     return instance_layout(funinstance_ptr);
@@ -207,15 +185,6 @@ static inline lispobj fdefn_callee_lispobj(struct fdefn *fdefn) {
 
 #endif
 
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-#include "genesis/layout.h"
-#define LAYOUT_SIZE (sizeof (struct layout)/N_WORD_BYTES)
-/// First 4 layouts: T, STRUCTURE-OBJECT, LAYOUT, FUNCTION
-/// (These #defines ought to be emitted by genesis)
-#define LAYOUT_OF_LAYOUT   ((FIXEDOBJ_SPACE_START+2*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
-#define LAYOUT_OF_FUNCTION ((FIXEDOBJ_SPACE_START+3*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
-#endif
-
 boolean valid_widetag_p(unsigned char widetag);
 
 #endif /* _GC_INTERNAL_H_ */
