@@ -760,7 +760,7 @@ fake_foreign_function_call(os_context_t *context)
     check_blockables_blocked_or_lose(0);
 
     /* Get current Lisp state from context. */
-#if defined(LISP_FEATURE_ARM) && !defined(LISP_FEATURE_GENCGC)
+#if (defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_RISCV)) && !defined(LISP_FEATURE_GENCGC)
     dynamic_space_free_pointer = SymbolValue(ALLOCATION_POINTER, thread);
 #endif
 #ifdef reg_ALLOC
@@ -804,8 +804,9 @@ fake_foreign_function_call(os_context_t *context)
     context_index =
         fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
-    if (context_index >= MAX_INTERRUPTS) {
-        lose("maximum interrupt nesting depth (%d) exceeded\n", MAX_INTERRUPTS);
+    if (context_index >= (MAX_INTERRUPTS-THREAD_HEADER_SLOTS)) {
+        lose("maximum interrupt nesting depth (%d) exceeded\n",
+             MAX_INTERRUPTS-THREAD_HEADER_SLOTS);
     }
 
     bind_variable(FREE_INTERRUPT_CONTEXT_INDEX,
@@ -874,7 +875,7 @@ undo_fake_foreign_function_call(os_context_t __attribute__((unused)) *context)
      * not updating them. */
     thread->pseudo_atomic_bits = 0;
 #endif
-#if defined(LISP_FEATURE_ARM) && !defined(LISP_FEATURE_GENCGC)
+#if (defined(LISP_FEATURE_ARM) || defined (LISP_FEATURE_RISCV)) && !defined(LISP_FEATURE_GENCGC)
     SetSymbolValue(ALLOCATION_POINTER, dynamic_space_free_pointer, thread);
 #endif
 }
@@ -895,13 +896,7 @@ interrupt_internal_error(os_context_t *context, boolean continuable)
         lose("internal error too early in init, can't recover\n");
     }
 
-#ifndef LISP_FEATURE_SB_SAFEPOINT
-    unblock_gc_signals(0, 0);
-#endif
-
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
-#endif
 
 #if defined(LISP_FEATURE_LINUX) && defined(LISP_FEATURE_MIPS)
     /* Workaround for blocked SIGTRAP. */
@@ -1255,8 +1250,8 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
     if ((read_TLS(INTERRUPTS_ENABLED,thread) == NIL) ||
         in_leaving_without_gcing_race_p(thread)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%x,%d): deferred (RACE=%d)\n",
-                      (unsigned int)handler,signal,
+                      "/maybe_defer_handler(%p,%d): deferred (RACE=%d)\n",
+                      handler,signal,
                       in_leaving_without_gcing_race_p(thread)));
         store_signal_data_for_later(data,handler,signal,info,context);
         write_TLS(INTERRUPT_PENDING, T,thread);
@@ -1268,8 +1263,8 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
      * may succeed even when context is null (gencgc alloc()) */
     if (arch_pseudo_atomic_atomic(context)) {
         FSHOW_SIGNAL((stderr,
-                      "/maybe_defer_handler(%x,%d): deferred(PA)\n",
-                      (unsigned int)handler,signal));
+                      "/maybe_defer_handler(%p,%d): deferred(PA)\n",
+                      handler,signal));
         store_signal_data_for_later(data,handler,signal,info,context);
         arch_set_pseudo_atomic_interrupted(context);
         check_interrupt_context_or_lose(context);
@@ -1279,8 +1274,8 @@ maybe_defer_handler(void *handler, struct interrupt_data *data,
     check_interrupt_context_or_lose(context);
 
     FSHOW_SIGNAL((stderr,
-                  "/maybe_defer_handler(%x,%d): not deferred\n",
-                  (unsigned int)handler,signal));
+                  "/maybe_defer_handler(%p,%d): not deferred\n",
+                  handler,signal));
     return 0;
 }
 
@@ -1464,7 +1459,11 @@ interrupt_handle_now_handler(int signal, siginfo_t *info, void *void_context)
 extern int *context_eflags_addr(os_context_t *context);
 #endif
 
+#ifdef CALL_INTO_LISP
+#define call_into_lisp ((lispobj(*)(lispobj fun, lispobj *args, int nargs))SYMBOL(CALL_INTO_LISP)->value)
+#else
 extern lispobj call_into_lisp(lispobj fun, lispobj *args, int nargs);
+#endif
 extern void post_signal_tramp(void);
 extern void call_into_lisp_tramp(void);
 
@@ -1479,7 +1478,7 @@ arrange_return_to_c_function(os_context_t *context,
 #endif
 #if !(defined(LISP_FEATURE_X86) || defined(LISP_FEATURE_X86_64))
     void * fun=native_pointer(function);
-    void *code = &(((struct simple_fun *) fun)->code);
+    void *code = &(((struct simple_fun *) fun)->insts);
 #endif
 
     /* Build a stack frame showing `interrupted' so that the
@@ -1651,7 +1650,7 @@ arrange_return_to_c_function(os_context_t *context,
     *os_context_npc_addr(context) =
         4 + *os_context_pc_addr(context);
 #endif
-#if defined(LISP_FEATURE_SPARC) || defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_ARM64)
+#if defined(LISP_FEATURE_SPARC) || defined(LISP_FEATURE_ARM) || defined(LISP_FEATURE_ARM64) || defined(LISP_FEATURE_RISCV)
     *os_context_register_addr(context,reg_CODE) =
         (os_context_register_t)((char*)fun + FUN_POINTER_LOWTAG);
 #endif
@@ -2234,9 +2233,9 @@ handle_memory_fault_emulation_trap(os_context_t *context)
 #endif /* C_STACK_IS_CONTROL_STACK */
     /* On x86oids, we're in handle_memory_fault_emulation_trap().
      * On real computers, we're still in lisp_memory_fault_error(). */
-#ifndef LISP_FEATURE_SB_SAFEPOINT
-    unblock_gc_signals(0, 0);
-#endif
+
+    thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
+
     DX_ALLOC_SAP(context_sap, context);
     DX_ALLOC_SAP(fault_address_sap, addr);
     funcall2(StaticSymbolFunction(MEMORY_FAULT_ERROR),
@@ -2250,13 +2249,9 @@ unhandled_trap_error(os_context_t *context)
 {
     DX_ALLOC_SAP(context_sap, context);
     fake_foreign_function_call(context);
-#ifndef LISP_FEATURE_SB_SAFEPOINT
-    unblock_gc_signals(0, 0);
-#endif
 
-#if !defined(LISP_FEATURE_WIN32) || defined(LISP_FEATURE_SB_THREAD)
     thread_sigmask(SIG_SETMASK, os_context_sigmask_addr(context), 0);
-#endif
+
     funcall1(StaticSymbolFunction(UNHANDLED_TRAP_ERROR), context_sap);
     lose("UNHANDLED-TRAP-ERROR fell through");
 }
