@@ -23,6 +23,7 @@
 #include "sbcl.h"
 #include "print.h"
 #include "runtime.h"
+#include "code.h"
 #include "gc-internal.h"
 #include <stdarg.h>
 #include "thread.h"              /* genesis/primitive-objects.h needs this */
@@ -556,10 +557,15 @@ static void print_slots(char **slots, int count, lispobj *ptr)
 {
     while (count-- > 0) {
         if (*slots) {
-            print_obj(*slots++, *ptr++);
+            // kludge for half-lispword sized slot
+            print_obj(*slots,
+                      (N_WORD_BYTES == 8 && !strcmp(*slots, "boxed_size: "))
+                      ? *ptr & 0xFFFFFFFF : *ptr);
+            slots++;
         } else {
-            print_obj("???: ", *ptr++);
+            print_obj("???: ", *ptr);
         }
+        ptr++;
     }
 }
 
@@ -734,7 +740,7 @@ static void print_otherptr(lispobj obj)
 
     case CODE_HEADER_WIDETAG:
         // ptr was already bumped up
-        count = code_header_words(((struct code*)(ptr-1))->header);
+        count = code_header_words((struct code*)(ptr-1));
         for_each_simple_fun(fun_index, fun, (struct code*)(ptr-1), 0, {
             sprintf(buffer, "f[%d]: ", fun_index);
             print_obj(buffer, make_lispobj(fun,FUN_POINTER_LOWTAG));
@@ -743,10 +749,7 @@ static void print_otherptr(lispobj obj)
         break;
 
     case SIMPLE_FUN_WIDETAG:
-        print_obj("code: ",
-                  make_lispobj(native_pointer((lispobj)(ptr-1))
-                               -(HeaderValue(header)&0xFFFF),
-                               OTHER_POINTER_LOWTAG));
+        print_obj("code: ", fun_code_tagged(ptr-1));
         print_slots(simple_fun_slots,
                     sizeof simple_fun_slots/sizeof(char*)-1, ptr);
         break;
