@@ -117,9 +117,8 @@ static void add_to_layer(lispobj* obj, int wordindex,
 /// otherwise return obj directly.
 static lispobj canonical_obj(lispobj obj)
 {
-    if (functionp(obj) && widetag_of((lispobj*)(obj-FUN_POINTER_LOWTAG)) == SIMPLE_FUN_WIDETAG)
-        return make_lispobj(fun_code_header(obj-FUN_POINTER_LOWTAG),
-                            OTHER_POINTER_LOWTAG);
+    if (functionp(obj) && widetag_of(FUNCTION(obj)) == SIMPLE_FUN_WIDETAG)
+        return fun_code_tagged(FUNCTION(obj));
     return obj;
 }
 
@@ -130,7 +129,7 @@ static lispobj canonical_obj(lispobj obj)
 static int find_ref(lispobj* source, lispobj target)
 {
     lispobj layout, bitmap;
-    int scan_limit, i, j;
+    int scan_limit, i;
 
     lispobj header = *source;
     if (is_cons_half(header)) {
@@ -162,10 +161,6 @@ static int find_ref(lispobj* source, lispobj target)
         break;
 #endif
     case CODE_HEADER_WIDETAG:
-        for_each_simple_fun(i, function_ptr, (struct code*)source, 0, {
-            int wordindex = &function_ptr->name - source;
-            for (j=0; j<4; ++j) check_ptr(wordindex+j, source[wordindex+j]);
-        })
         scan_limit = code_header_words((struct code*)source);
         break;
     case FDEFN_WIDETAG:
@@ -415,7 +410,7 @@ static inline lispobj decode_pointer(uint32_t encoding)
 
 static struct simple_fun* simple_fun_from_pc(char* pc)
 {
-    struct code* code = (struct code*)component_ptr_from_pc((lispobj*)pc);
+    struct code* code = (struct code*)component_ptr_from_pc(pc);
     if (!code) return 0;
     struct simple_fun* prev_fun = 0;
     for_each_simple_fun(i, fun, code, 1, {
@@ -602,10 +597,11 @@ static int trace1(lispobj object,
             struct simple_fun* fun = simple_fun_from_pc(thread_pc);
             if (fun) {
                 fprintf(file, "fun=%p", (void*)make_lispobj(fun, FUN_POINTER_LOWTAG));
-                if (is_lisp_pointer(fun->name) &&
-                    widetag_of(native_pointer(fun->name)) == SYMBOL_WIDETAG) {
+                lispobj fun_name = simple_fun_name(fun);
+                if (is_lisp_pointer(fun_name) &&
+                    widetag_of(native_pointer(fun_name)) == SYMBOL_WIDETAG) {
                     fprintf(file, "=");
-                    show_lstring(VECTOR(SYMBOL(fun->name)->name), 0, file);
+                    show_lstring(VECTOR(SYMBOL(fun_name)->name), 0, file);
                 }
             } else if (thread_pc)
                 fprintf(file, "pc=%p", thread_pc);
@@ -695,7 +691,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                           struct scan_state* ss)
 {
     lispobj layout, bitmap;
-    sword_t nwords, scan_limit, i, j;
+    sword_t nwords, scan_limit, i;
     uword_t n_objects = 0, n_scanned_words = 0,
             n_immediates = 0, n_pointers = 0;
 
@@ -733,10 +729,6 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             break;
 #endif
         case CODE_HEADER_WIDETAG:
-            for_each_simple_fun(i, function_ptr, (struct code*)where, 0, {
-                int wordindex = &function_ptr->name - where;
-                for (j=0; j<4; ++j) check_ptr(where[wordindex+j]);
-            })
             scan_limit = code_header_words((struct code*)where);
             break;
         case FDEFN_WIDETAG:
@@ -770,7 +762,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             if (!(other_immediate_lowtag_p(widetag) && lowtag_for_widetag[widetag>>2]))
               lose("Unknown widetag %x\n", widetag);
             // Skip irrelevant objects.
-            if (unboxed_obj_widetag_p(widetag) ||
+            if (leaf_obj_widetag_p(widetag) ||
                 (widetag == WEAK_POINTER_WIDETAG) || /* do not follow! */
                 // These numeric types contain pointers, but are uninteresting.
                 (widetag == COMPLEX_WIDETAG) ||
@@ -980,7 +972,7 @@ static struct instance* find_thread(os_thread_t os_thread, lispobj tree)
 struct vector* lisp_thread_name(os_thread_t os_thread)
 {
     static unsigned int hint;
-    lispobj* sym = find_symbol("*ALL-THREADS*", "SB-THREAD", &hint);
+    lispobj* sym = find_symbol("*ALL-THREADS*", find_package("SB-THREAD"), &hint);
     if (sym) {
         struct instance* lisp_thread =
             find_thread(os_thread, ((struct symbol*)sym)->value);
