@@ -44,7 +44,7 @@
 #define BREAKPOINT_WIDTH 1
 #endif
 unsigned int cpuid_fn1_ecx;
-unsigned int avx_supported = 0;
+int avx_supported = 0, avx2_supported = 0;
 
 static void cpuid(unsigned info, unsigned subinfo,
                   unsigned *eax, unsigned *ebx, unsigned *ecx, unsigned *edx)
@@ -68,26 +68,9 @@ static void cpuid(unsigned info, unsigned subinfo,
 
 static void xgetbv(unsigned *eax, unsigned *edx)
 {
-  __asm__("xgetbv;"
-          :"=a" (*eax), "=d" (*edx)
-          : "c" (0));
-}
-
-void arch_init(void)
-{
-  unsigned int eax, ebx, ecx, edx;
-
-  cpuid(0, 0, &eax, &ebx, &ecx, &edx);
-  if (eax >= 1) { // see if we can execute basic id function 1
-      unsigned avx_mask = 0x18000000; // OXSAVE and AVX
-      cpuid(1, 0, &eax, &ebx, &ecx, &edx);
-      cpuid_fn1_ecx = ecx;
-      if ((ecx & avx_mask) == avx_mask) {
-          xgetbv(&eax, &edx);
-          if ((eax & 0x06) == 0x06) // YMM and XMM
-              avx_supported = 1;
-      }
-  }
+    __asm__("xgetbv;"
+            :"=a" (*eax), "=d" (*edx)
+            : "c" (0));
 }
 
 #define VECTOR_FILL_T "VECTOR-FILL/T"
@@ -96,9 +79,27 @@ void arch_init(void)
 // Using fixed offsets and bytes is no worse than what we do elsewhere.
 void tune_asm_routines_for_microarch(void)
 {
+    unsigned int eax, ebx, ecx, edx;
+
+    cpuid(0, 0, &eax, &ebx, &ecx, &edx);
+    if (eax >= 1) { // see if we can execute basic id function 1
+        unsigned avx_mask = 0x18000000; // OXSAVE and AVX
+        cpuid(1, 0, &eax, &ebx, &ecx, &edx);
+        cpuid_fn1_ecx = ecx;
+        if ((ecx & avx_mask) == avx_mask) {
+            xgetbv(&eax, &edx);
+            if ((eax & 0x06) == 0x06) { // YMM and XMM
+                avx_supported = 1;
+                cpuid(7, 0, &eax, &ebx, &ecx, &edx);
+                if  (ebx & 0x20)  {
+                    avx2_supported = 1;
+                }
+            }
+        }
+    }
+    SetSymbolValue(CPUID_FN1_ECX, (lispobj)make_fixnum(cpuid_fn1_ecx), 0);
     // I don't know if this works on Windows
 #ifndef _MSC_VER
-    unsigned int eax, ebx, ecx, edx;
     cpuid(0, 0, &eax, &ebx, &ecx, &edx);
     if (eax >= 7) {
         cpuid(7, 0, &eax, &ebx, &ecx, &edx);
@@ -625,7 +626,7 @@ static boolean instrumentp(uword_t* sp, uword_t** pc, uword_t* old_word)
 
 // logical index 'index' in the metadata array stores the code component
 // and pc-offset relative to the component base address
-static void record_pc(uword_t* pc, unsigned int index, boolean sizedp)
+static void record_pc(char* pc, unsigned int index, boolean sizedp)
 {
     lispobj *code = component_ptr_from_pc(pc);
     if (!code) {
@@ -644,7 +645,7 @@ static void record_pc(uword_t* pc, unsigned int index, boolean sizedp)
         ensure_region_closed(code_region);
         int ret = thread_mutex_unlock(&free_pages_lock);
         gc_assert(ret == 0);
-        code = component_ptr_from_pc((lispobj*)pc);
+        code = component_ptr_from_pc(pc);
     }
 #endif
     struct vector* v = VECTOR(alloc_profile_data);
@@ -680,7 +681,7 @@ allocation_tracker_counted(uword_t* sp)
         if (!__sync_bool_compare_and_swap(pc, word_at_pc, new_inst))
             lose("alloc profiler failed to rewrite instruction @ %p", pc);
         if (index != 2)
-            record_pc(pc, index, 0);
+            record_pc((char*)pc, index, 0);
     }
     int __attribute__((unused)) ret = thread_mutex_unlock(&alloc_profiler_lock);
     gc_assert(ret == 0);
@@ -719,7 +720,7 @@ allocation_tracker_sized(uword_t* sp)
             !__sync_bool_compare_and_swap(pc,   word_at_pc,    new_inst1))
             lose("alloc profiler failed to rewrite instructions @ %p", pc);
         if (index != 0) // can't record a PC for the overflow counts
-            record_pc(pc, index, 1);
+            record_pc((char*)pc, index, 1);
     }
     int __attribute__((unused)) ret = thread_mutex_unlock(&alloc_profiler_lock);
     gc_assert(ret == 0);
