@@ -206,14 +206,15 @@
                   :skipped-on (not (and :x86-64 :immobile-space)))
   (let ((addr-of-pathname-layout
          (write-to-string
-          (sb-kernel:get-lisp-obj-address (sb-kernel:find-layout 'pathname))
+          (sb-kernel:get-lisp-obj-address
+           (sb-kernel:find-layout 'hash-table))
           :base 16 :radix t))
         (count 0))
     ;; The constant should appear in two CMP instructions
     (dolist (line (split-string
                    (with-output-to-string (s)
                      (let ((sb-disassem:*disassem-location-column-width* 0))
-                       (disassemble 'pathnamep :stream s)))
+                       (disassemble 'hash-table-p :stream s)))
                    #\newline))
       (when (and (search "CMP" line) (search addr-of-pathname-layout line))
         (incf count)))
@@ -2290,4 +2291,74 @@
       `(lambda (a)
          (let ((v4 (the (or (single-float (1.0) (3.0)) (single-float 4.0 5.0)) a)))
            (incf v4 1.0)))
-    ((4.0) 5.0)))
+      ((4.0) 5.0)))
+
+(with-test (:name :derive-array-rank-negation)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare ((not (simple-array * (* *))) a))
+      (eql (array-rank a) 2))
+   (((make-array '(2 2) :adjustable t)) t))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare ((not (simple-array fixnum (* *))) a))
+      (eql (array-rank a) 2))
+   (((make-array '(2 2))) t))
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (declare ((not (and (array * (* *)) (not simple-array))) a))
+      (eql (array-rank a) 2))
+   (((make-array '(2 2))) t)))
+
+(with-test (:name :derive-array-rank-negation.2)
+  (assert
+   (equal (sb-kernel:%simple-fun-type
+           (checked-compile
+            '(lambda (x)
+              (declare ((and simple-array
+                         (not (simple-array * (* *))))
+                        x))
+              (eql (array-rank x) 2))))
+          '(function ((and simple-array (not (simple-array * (* *)))))
+            (values null &optional)))))
+
+(with-test (:name :known-fun-no-fdefn)
+  (assert (not (ctu:find-named-callees
+                (checked-compile
+                 '(lambda () #'+))))))
+
+(with-test (:name :double-float-p-weakening)
+  (checked-compile-and-assert
+   (:optimize '(:speed 3 :safety 1))
+   '(lambda (x)
+     (declare (double-float x))
+     x)
+   ((0.0) (condition 'type-error))
+      ((1d0) 1d0)))
+
+#+sb-unicode
+(with-test (:name :base-char-p-constraint-propagation)
+  (assert
+   (equal (sb-kernel:%simple-fun-type
+           (checked-compile
+            '(lambda (x)
+              (if (sb-kernel:base-char-p x)
+                  (characterp x)
+                  t))))
+          '(function (t) (values (member t) &optional)))))
+
+(declaim (inline inline-fun-arg-mismatch))
+(defun inline-fun-arg-mismatch (x)
+  (declare (optimize (debug 0)))
+  x)
+
+(with-test (:name :inline-fun-arg-mismatch)
+  (checked-compile-and-assert
+      (:allow-warnings '(or sb-int:local-argument-mismatch
+                         #+interpreter simple-warning)) ;; why?
+      '(lambda ()
+        (multiple-value-call #'inline-fun-arg-mismatch 1 2))
+    (() (condition 'program-error))))
