@@ -9,15 +9,6 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package :cl-user)
-
-;;; +MAGIC-HASH-VECTOR-VALUE+ is used to mark empty entries in the slot
-;;; HASH-VECTOR of hash tables. It must be a value outside of the range
-;;; of SXHASH. The range of SXHASH is the non-negative fixnums.
-(with-test (:name :magic-hash-vector-value)
-  (assert (not (typep sb-impl::+magic-hash-vector-value+
-                      '(and fixnum unsigned-byte)))))
-
 ;;; The return value of SXHASH on non-string/bitvector arrays should not
 ;;; change when the contents of the array change.
 (with-test (:name (sxhash array :independent-of-contents))
@@ -45,11 +36,11 @@
   ;; but it's best to verify the assumption that each cons bumps the count
   ;; by 1, lest it be violated in a way that affects the quality of CTYPE
   ;; hashes.
-  (let ((win 0) (n-trials 10) (prev (sb-impl::address-based-counter-val)))
+  (let ((win 0) (n-trials 10) (prev (sb-int:address-based-counter-val)))
     (dotimes (i n-trials)
       (declare (notinline cons)) ; it's flushable, but don't flush it
       (cons 1 2)
-      (let ((ptr (sb-impl::address-based-counter-val)))
+      (let ((ptr (sb-int:address-based-counter-val)))
         (when (= ptr (1+ prev))
           (incf win))
         (setq prev ptr)))
@@ -130,3 +121,42 @@
                      (complex sb-ext:single-float-negative-infinity 0)
                      (complex sb-ext:double-float-negative-infinity 0)))
       (assert (eql (gethash v table) -1)))))
+
+(with-test (:name (:hash equalp pathname))
+  (let* ((map (make-hash-table :test 'equalp))
+         (key  #P"some/path/"))
+    (setf (gethash key map) "my-value")
+    (format (make-broadcast-stream) "Printing: ~A~%" key)
+    (assert (remhash key map))
+    (assert (= 0 (hash-table-count map)))))
+
+(with-test (:name :clrhash-clears-rehash-p)
+  (let ((tbl (make-hash-table)))
+    (dotimes (i 10)
+      (setf (gethash (cons 'foo (gensym)) tbl) 1))
+    (gc)
+    ;; The need-to-rehash bit is set
+    (assert (eql 1 (svref (sb-impl::hash-table-table tbl) 1)))
+    (clrhash tbl)
+    ;; The need-to-rehash bit is not set
+    (assert (eql 0 (svref (sb-impl::hash-table-table tbl) 1)))))
+
+(with-test (:name :sxhash-signed-floating-point-zeros)
+  (assert (not (eql (sxhash -0f0) (sxhash 0f0))))
+  (assert (not (eql (sxhash -0d0) (sxhash 0d0)))))
+
+(with-test (:name :sxhash-simple-bit-vector)
+  (let (hashes)
+    (let ((v (make-array sb-vm:n-word-bits :element-type 'bit)))
+      (dotimes (i sb-vm:n-word-bits)
+        (setf (aref v i) 1)
+        (push (sxhash v) hashes)
+        (setf (aref v i) 0)))
+    (assert (= (length (remove-duplicates hashes)) sb-vm:n-word-bits))))
+
+(with-test (:name :eq-hash-nonpointers-not-address-sensitive)
+  (let ((tbl (make-hash-table :test 'eq)))
+    (setf (gethash #\a tbl) 1)
+    #+64-bit (setf (gethash 1.0f0 tbl) 1) ; single-float is a nonpointer
+    (let ((data (sb-kernel:get-header-data (sb-impl::hash-table-table tbl))))
+      (assert (zerop data))))) ; not sb-vm:vector-valid-hashing-subtype
