@@ -11,10 +11,6 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package "CL-USER")
-
-(load "compiler-test-util.lisp")
-
 (assert (equal (symbol-name '#:|fd\sA|) "fdsA"))
 
 ;;; Prior to sbcl-0.7.2.10, SBCL disobeyed the ANSI requirements on
@@ -212,7 +208,7 @@
       (assert (null (loop for c across standard-chars append (frob c)))))))
 
 (with-test (:name :copy-readtable-with-unicode-macro
-                  :skipped-on '(not :sb-unicode))
+                  :skipped-on (not :sb-unicode))
   (let ((rt (copy-readtable)))
     (set-macro-character (code-char #x100fa) #'error nil rt)
     (assert (plusp (hash-table-count (sb-impl::character-macro-hash-table rt))))
@@ -336,7 +332,6 @@
                  (read-from-string "sb-c::(a sb-kernel::(x y) b)")))
   (assert (equal '(cl-user::yes-this-is-sbcl)
                  (read-from-string "cl-user::(#+sbcl yes-this-is-sbcl)")))
-  #+sb-package-locks
   (assert (eq :violated!
               (handler-case
                   (read-from-string "cl::'foo")
@@ -365,7 +360,7 @@
 ;;  - multiple-value-call in WITH-CHAR-MACRO-RESULT
 ;;  - the initial cons cell in READ-LIST
 (with-test (:name :read-does-not-cons-per-se
-                  :skipped-on '(:or :interpreter (:not :x86-64)))
+                  :skipped-on (:or :interpreter (:not :x86-64)))
   (flet ((test-reading (string)
            (let ((s (make-string-input-stream string)))
              (read s) ; once outside the loop, to make A-SYMBOL
@@ -408,8 +403,7 @@
 
 ;;; The WITH-FAST-READ-BYTE macro accidentally left the package lock
 ;;; of FAST-READ-BYTE disabled during its body.
-(with-test (:name :fast-read-byte-package-lock
-            :skipped-on '(not :sb-package-locks))
+(with-test (:name :fast-read-byte-package-lock)
   (let ((fun
          ;; Suppress the compiler output to avoid noise when running the
          ;; test. (There are a warning and an error about the package
@@ -553,3 +547,33 @@
       (expect nil
               '(simple-array character (9))
               '(simple-array character (9))))))
+
+(with-test (:name :rational-rdff)
+  (with-standard-io-syntax
+    (let ((*read-default-float-format* 'rational))
+      (macrolet ((assert-read-eqlity (x y)
+                   `(assert (eql (read-from-string ,x) ,y)))
+                 (assert-print-string= (x y)
+                   `(assert (string= (prin1-to-string ,x) ,y))))
+        (assert-read-eqlity "1.0" 1)
+        (assert-read-eqlity "-1.0e0" -1)
+        (assert-read-eqlity "0.1" 1/10)
+        (assert-read-eqlity "-0.1e0" -1/10)
+        (assert-read-eqlity "0.1d0" 0.1d0)
+        (assert-read-eqlity "-0.1f0" -0.1f0)
+
+        (assert-print-string= 1 "1")
+        (assert-print-string= 1/10 "1/10")
+        (assert-print-string= 0.1f0 "0.1f0")
+        (assert-print-string= 0.1d0 "0.1d0")))))
+
+(with-test (:name :rational-exponent)
+  (with-standard-io-syntax
+    (macrolet ((assert-read-eqlity (x y)
+                 `(assert (eql (read-from-string ,x) ,y))))
+      (assert-read-eqlity "1.0r0" 1)
+      (assert-read-eqlity "1.0r5" 100000)
+      (assert-read-eqlity "0.1r0" 1/10)
+      (assert-read-eqlity "0.333R0" 333/1000)
+      (assert-read-eqlity "1R-3" 1/1000)
+      (assert-read-eqlity ".1R2" 10))))
